@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,7 +37,21 @@ public:
      * @param files output map that associates the size of each file with its name
      * @param path root path
      */
-    static void recursiveAddFiles(std::map<std::string, std::size_t> &files, const std::string &path);
+    static void recursiveAddFiles(std::map<std::string, std::size_t> &files, const std::string &path) {
+        const std::set<std::string> extra;
+        recursiveAddFiles(files, path, extra);
+    }
+
+    /**
+     * @brief Recursively add source files to a map.
+     * Add source files from given directory and all subdirectries to the
+     * given map. Only files with accepted extensions
+     * (*.c;*.cpp;*.cxx;*.c++;*.cc;*.txx) are added.
+     * @param files output map that associates the size of each file with its name
+     * @param path root path
+     * @param extra Extra file extensions
+     */
+    static void recursiveAddFiles(std::map<std::string, std::size_t> &files, const std::string &path, const std::set<std::string> &extra);
 
     /**
      * @brief Is given path a directory?
@@ -56,7 +70,8 @@ public:
 
     static void recursiveAddFiles2(std::set<std::string> &seen_paths,
                                    std::map<std::string, std::size_t> &files,
-                                   const std::string &path);
+                                   const std::string &path,
+                                   const std::set<std::string> &extra);
 #endif
 };
 
