@@ -22,7 +22,7 @@
         <location filename="about.ui" line="81"/>
         <source>Copyright © 2007-2013 Daniel Marjamäki and cppcheck team.</source>
         <oldsource>Copyright © 2007-2012 Daniel Marjamäki and cppcheck team.</oldsource>
-        <translation type="unfinished">Copyright (C) 2007-2010 Daniel Marjamäki and cppcheck team.</translation>
+        <translation>Copyright (C) 2007-2013 Daniel Marjamäki and cppcheck team.</translation>
     </message>
     <message>
         <location filename="about.ui" line="91"/>
@@ -107,7 +107,7 @@ Parametrar: -l(line) (file)</translation>
     <message>
         <location filename="applicationdialog.cpp" line="78"/>
         <source>You must specify a name, a path and optionally parameters for the application!</source>
-        <translation type="unfinished"></translation>
+        <translation>Du måste ange namn, sökväg samt eventuellt parametrar för applikationen!</translation>
     </message>
     <message>
         <source>You must specify a name, a path and parameters for the application!</source>
@@ -185,11 +185,11 @@ Parametrar: -l(line) (file)</translation>
         <location filename="mainwindow.cpp" line="394"/>
         <location filename="mainwindow.cpp" line="457"/>
         <location filename="mainwindow.cpp" line="479"/>
-        <location filename="mainwindow.cpp" line="672"/>
-        <location filename="mainwindow.cpp" line="777"/>
-        <location filename="mainwindow.cpp" line="897"/>
-        <location filename="mainwindow.cpp" line="1065"/>
-        <location filename="mainwindow.cpp" line="1146"/>
+        <location filename="mainwindow.cpp" line="679"/>
+        <location filename="mainwindow.cpp" line="784"/>
+        <location filename="mainwindow.cpp" line="904"/>
+        <location filename="mainwindow.cpp" line="1072"/>
+        <location filename="mainwindow.cpp" line="1153"/>
         <source>Cppcheck</source>
         <translation>Cppcheck</translation>
     </message>
@@ -221,12 +221,12 @@ Parametrar: -l(line) (file)</translation>
     <message>
         <location filename="main.ui" line="131"/>
         <source>C++ standard</source>
-        <translation type="unfinished"></translation>
+        <translation>C++ standard</translation>
     </message>
     <message>
         <location filename="main.ui" line="138"/>
         <source>C standard</source>
-        <translation type="unfinished"></translation>
+        <translation>C standard</translation>
     </message>
     <message>
         <location filename="main.ui" line="157"/>
@@ -345,7 +345,7 @@ Parametrar: -l(line) (file)</translation>
     <message>
         <location filename="main.ui" line="446"/>
         <source>Show S&amp;cratchpad...</source>
-        <translation type="unfinished"></translation>
+        <translation>Visa s&amp;cratchpad...</translation>
     </message>
     <message>
         <location filename="main.ui" line="496"/>
@@ -397,72 +397,72 @@ Parametrar: -l(line) (file)</translation>
     <message>
         <location filename="main.ui" line="566"/>
         <source>&amp;Filter</source>
-        <translation type="unfinished"></translation>
+        <translation>&amp;Filter</translation>
     </message>
     <message>
         <location filename="main.ui" line="569"/>
         <source>Filter results</source>
-        <translation type="unfinished"></translation>
+        <translation>Filtrera resultat</translation>
     </message>
     <message>
         <location filename="main.ui" line="585"/>
         <source>Windows 32-bit ANSI</source>
-        <translation type="unfinished"></translation>
+        <translation>Windows 32-bit ANSI</translation>
     </message>
     <message>
         <location filename="main.ui" line="593"/>
         <source>Windows 32-bit Unicode</source>
-        <translation type="unfinished"></translation>
+        <translation>Windows 32-bit Unicode</translation>
     </message>
     <message>
         <location filename="main.ui" line="601"/>
         <source>Unix 32-bit</source>
-        <translation type="unfinished"></translation>
+        <translation>Unix 32-bit</translation>
     </message>
     <message>
         <location filename="main.ui" line="609"/>
         <source>Unix 64-bit</source>
-        <translation type="unfinished"></translation>
+        <translation>Unix 64-bit</translation>
     </message>
     <message>
         <location filename="main.ui" line="617"/>
         <source>Windows 64-bit</source>
-        <translation type="unfinished"></translation>
+        <translation>Windows 64-bit</translation>
     </message>
     <message>
         <location filename="main.ui" line="625"/>
         <source>Platforms</source>
-        <translation type="unfinished"></translation>
+        <translation>Plattformar</translation>
     </message>
     <message>
         <location filename="main.ui" line="639"/>
         <source>C++11</source>
-        <translation type="unfinished"></translation>
+        <translation>C++11</translation>
     </message>
     <message>
         <location filename="main.ui" line="650"/>
         <source>C99</source>
-        <translation type="unfinished"></translation>
+        <translation>C99</translation>
     </message>
     <message>
         <location filename="main.ui" line="658"/>
         <source>Posix</source>
-        <translation type="unfinished"></translation>
+        <translation>Posix</translation>
     </message>
     <message>
         <location filename="main.ui" line="666"/>
         <source>C11</source>
-        <translation type="unfinished"></translation>
+        <translation>C11</translation>
     </message>
     <message>
         <location filename="main.ui" line="674"/>
         <source>C89</source>
-        <translation type="unfinished"></translation>
+        <translation>C89</translation>
     </message>
     <message>
         <location filename="main.ui" line="682"/>
         <source>C++03</source>
-        <translation type="unfinished"></translation>
+        <translation>C++03</translation>
     </message>
     <message>
         <location filename="main.ui" line="370"/>
@@ -472,7 +472,7 @@ Parametrar: -l(line) (file)</translation>
     <message>
         <location filename="main.ui" line="207"/>
         <source>Filter</source>
-        <translation type="unfinished"></translation>
+        <translation>Filter</translation>
     </message>
     <message>
         <location filename="main.ui" line="375"/>
@@ -604,45 +604,49 @@ Parametrar: -l(line) (file)</translation>
     <message>
         <location filename="mainwindow.cpp" line="74"/>
         <source>Quick Filter:</source>
-        <translation type="unfinished"></translation>
+        <translation>Snabbfilter:</translation>
     </message>
     <message>
         <location filename="mainwindow.cpp" line="458"/>
         <source>Found project file: %1
 
 Do you want to load this project file instead?</source>
-        <translation type="unfinished"></translation>
+        <translation>Hittade projektfil: %1
+
+Vill du ladda denna projektfil istället?</translation>
     </message>
     <message>
         <location filename="mainwindow.cpp" line="480"/>
         <source>Found project files from the directory.
 
 Do you want to proceed checking without using any of these project files?</source>
-        <translation type="unfinished"></translation>
+        <translation>Hittade projektfil(er) i mappen.
+
+Vill du fortsätta analysen utan att använda någon av dessa projektfiler?</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="819"/>
+        <location filename="mainwindow.cpp" line="826"/>
         <source>License</source>
         <translation>Licens</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="826"/>
+        <location filename="mainwindow.cpp" line="833"/>
         <source>Authors</source>
         <translation>Utvecklare</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="834"/>
+        <location filename="mainwindow.cpp" line="841"/>
         <source>XML files version 2 (*.xml);;XML files version 1 (*.xml);;Text files (*.txt);;CSV files (*.csv)</source>
         <oldsource>XML files (*.xml);;Text files (*.txt);;CSV files (*.csv)</oldsource>
         <translation>XML filer version 2 (*.xml);;XML filer version 1 (*.xml);;Text filer (*.txt);;CSV filer (*.csv)</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="836"/>
+        <location filename="mainwindow.cpp" line="843"/>
         <source>Save the report file</source>
         <translation>Spara rapport</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="688"/>
+        <location filename="mainwindow.cpp" line="695"/>
         <source>XML files (*.xml)</source>
         <translation>XML filer (*.xml)</translation>
     </message>
@@ -663,22 +667,24 @@ En trolig orsak är att inställningarna ändrats för olika Cppcheck versioner.
     <message>
         <location filename="mainwindow.cpp" line="409"/>
         <source>Select files to check</source>
-        <translation type="unfinished">Välj filer att kontrollera</translation>
+        <translation>Välj filer att kontrollera</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="673"/>
+        <location filename="mainwindow.cpp" line="680"/>
         <source>Current results will be cleared.
 
 Opening a new XML file will clear current results.Do you want to proceed?</source>
-        <translation type="unfinished"></translation>
+        <translation>Nuvarande resultat kommer rensas bort.
+
+När en ny XML fil öppnas så tas alla nuvarande resultat bort. Vill du fortsätta?</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="690"/>
+        <location filename="mainwindow.cpp" line="697"/>
         <source>Open the report file</source>
         <translation>Öppna rapportfilen</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="773"/>
+        <location filename="mainwindow.cpp" line="780"/>
         <source>Checking is running.
 
 Do you want to stop the checking and exit Cppcheck?.</source>
@@ -687,27 +693,27 @@ Do you want to stop the checking and exit Cppcheck?.</source>
 Vill du stoppa analysen och avsluta Cppcheck?</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="843"/>
+        <location filename="mainwindow.cpp" line="850"/>
         <source>XML files version 1 (*.xml)</source>
         <translation>XML filer version 1 (*.xml)</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="847"/>
+        <location filename="mainwindow.cpp" line="854"/>
         <source>XML files version 2 (*.xml)</source>
         <translation>XML filer version 2 (*.xml)</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="851"/>
+        <location filename="mainwindow.cpp" line="858"/>
         <source>Text files (*.txt)</source>
         <translation>Text filer (*.txt)</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="855"/>
+        <location filename="mainwindow.cpp" line="862"/>
         <source>CSV files (*.csv)</source>
         <translation>CSV filer (*.csv)</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="899"/>
+        <location filename="mainwindow.cpp" line="906"/>
         <source>Cppcheck - %1</source>
         <translation>Cppcheck - %1</translation>
     </message>
@@ -724,34 +730,34 @@ The user interface language has been reset to English. Open the Preferences-dial
 Språket har nollställts till Engelska. Öppna Preferences och välj något av de tillgängliga språken.</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="944"/>
-        <location filename="mainwindow.cpp" line="1027"/>
+        <location filename="mainwindow.cpp" line="951"/>
+        <location filename="mainwindow.cpp" line="1034"/>
         <source>Project files (*.cppcheck);;All files(*.*)</source>
         <translation>Projektfiler (*.cppcheck);;Alla filer(*.*)</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="946"/>
+        <location filename="mainwindow.cpp" line="953"/>
         <source>Select Project File</source>
         <translation>Välj projektfil</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="974"/>
-        <location filename="mainwindow.cpp" line="1041"/>
+        <location filename="mainwindow.cpp" line="981"/>
+        <location filename="mainwindow.cpp" line="1048"/>
         <source>Project:</source>
-        <translation type="unfinished">Projekt:</translation>
+        <translation>Projekt:</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="1029"/>
+        <location filename="mainwindow.cpp" line="1036"/>
         <source>Select Project Filename</source>
         <translation>Välj Projektfil</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="1066"/>
+        <location filename="mainwindow.cpp" line="1073"/>
         <source>No project file loaded</source>
         <translation>Inget projekt laddat</translation>
     </message>
     <message>
-        <location filename="mainwindow.cpp" line="1141"/>
+        <location filename="mainwindow.cpp" line="1148"/>
         <source>The project file
 
 %1
@@ -759,7 +765,13 @@ Språket har nollställts till Engelska. Öppna Preferences och välj något av
  could not be found!
 
 Do you want to remove the file from the recently used projects -list?</source>
-        <translation type="unfinished"></translation>
+        <translation>Projektfilen
+
+%1
+
+ kunde inte hittas!
+
+Vill du ta bort filen från &apos;senast använda projekt&apos;-listan?</translation>
     </message>
     <message>
         <location filename="translationhandler.cpp" line="45"/>
@@ -774,7 +786,7 @@ Do you want to remove the file from the recently used projects -list?</source>
     <message>
         <location filename="translationhandler.cpp" line="42"/>
         <source>Chinese (Simplified)</source>
-        <translation type="unfinished"></translation>
+        <translation>Kinesiska (Förenklad)</translation>
     </message>
     <message>
         <location filename="translationhandler.cpp" line="43"/>
@@ -789,17 +801,17 @@ Do you want to remove the file from the recently used projects -list?</source>
     <message>
         <location filename="translationhandler.cpp" line="48"/>
         <source>Italian</source>
-        <translation type="unfinished"></translation>
+        <translation>Italienska</translation>
     </message>
     <message>
         <location filename="translationhandler.cpp" line="50"/>
         <source>Korean</source>
-        <translation type="unfinished"></translation>
+        <translation>Koreanska</translation>
     </message>
     <message>
         <location filename="translationhandler.cpp" line="53"/>
         <source>Spanish</source>
-        <translation type="unfinished"></translation>
+        <translation>Spanska</translation>
     </message>
     <message>
         <location filename="translationhandler.cpp" line="54"/>
@@ -828,7 +840,7 @@ Do you want to remove the file from the recently used projects -list?</source>
         <translation>Serbiska</translation>
     </message>
     <message>
-        <location filename="main.cpp" line="87"/>
+        <location filename="main.cpp" line="91"/>
         <source>Cppcheck GUI.
 
 Syntax:
@@ -840,12 +852,22 @@ Options:
     -l &lt;file&gt;      Open given results xml file
     -d &lt;directory&gt; Specify the directory that was checked to generate the results xml specified with -l
     -v, --version  Show program version</source>
-        <translation type="unfinished"></translation>
+        <translation>Cppcheck GUI.
+
+Syntax:
+    cppcheck-gui [OPTIONS] [files or paths]
+
+Options:
+    -h, --help     Print this help
+    -p &lt;file&gt;      Open given project file and start checking it
+    -l &lt;file&gt;      Open given results xml file
+    -d &lt;directory&gt; Specify the directory that was checked to generate the results xml specified with -l
+    -v, --version  Show program version</translation>
     </message>
     <message>
-        <location filename="main.cpp" line="99"/>
+        <location filename="main.cpp" line="103"/>
         <source>Cppcheck GUI - Command line parameters</source>
-        <translation type="unfinished"></translation>
+        <translation>Cppcheck GUI - Command line parameters</translation>
     </message>
 </context>
 <context>
@@ -853,32 +875,32 @@ Options:
     <message>
         <location filename="platforms.cpp" line="37"/>
         <source>Built-in</source>
-        <translation type="unfinished"></translation>
+        <translation>Generell</translation>
     </message>
     <message>
         <location filename="platforms.cpp" line="38"/>
         <source>Unix 32-bit</source>
-        <translation type="unfinished"></translation>
+        <translation>Unix 32-bit</translation>
     </message>
     <message>
         <location filename="platforms.cpp" line="39"/>
         <source>Unix 64-bit</source>
-        <translation type="unfinished"></translation>
+        <translation>Unix 64-bit</translation>
     </message>
     <message>
         <location filename="platforms.cpp" line="40"/>
         <source>Windows 32-bit ANSI</source>
-        <translation type="unfinished"></translation>
+        <translation>Windows 32-bit ANSI</translation>
     </message>
     <message>
         <location filename="platforms.cpp" line="41"/>
         <source>Windows 32-bit Unicode</source>
-        <translation type="unfinished"></translation>
+        <translation>Windows 32-bit Unicode</translation>
     </message>
     <message>
         <location filename="platforms.cpp" line="42"/>
         <source>Windows 64-bit</source>
-        <translation type="unfinished"></translation>
+        <translation>Windows 64-bit</translation>
     </message>
 </context>
 <context>
@@ -915,7 +937,7 @@ Options:
     <message>
         <location filename="projectfile.ui" line="49"/>
         <source>Root:</source>
-        <translation type="unfinished"></translation>
+        <translation>Rot:</translation>
     </message>
     <message>
         <location filename="projectfile.ui" line="68"/>
@@ -957,17 +979,17 @@ Options:
     <message>
         <location filename="projectfile.ui" line="192"/>
         <source>Up</source>
-        <translation type="unfinished"></translation>
+        <translation>Upp</translation>
     </message>
     <message>
         <location filename="projectfile.ui" line="199"/>
         <source>Down</source>
-        <translation type="unfinished"></translation>
+        <translation>Ned</translation>
     </message>
     <message>
         <location filename="projectfile.ui" line="211"/>
         <source>Exclude</source>
-        <translation type="unfinished"></translation>
+        <translation>Exkludera</translation>
     </message>
     <message>
         <location filename="projectfile.ui" line="32"/>
@@ -990,7 +1012,7 @@ Options:
     <message>
         <location filename="projectfiledialog.cpp" line="232"/>
         <source>Select a directory to check</source>
-        <translation type="unfinished"></translation>
+        <translation>Välj mapp att analysera</translation>
     </message>
     <message>
         <location filename="projectfiledialog.cpp" line="272"/>
@@ -1003,22 +1025,22 @@ Options:
     <message>
         <location filename="translationhandler.cpp" line="29"/>
         <source>OK</source>
-        <translation type="unfinished"></translation>
+        <translation>OK</translation>
     </message>
     <message>
         <location filename="translationhandler.cpp" line="30"/>
         <source>Cancel</source>
-        <translation type="unfinished"></translation>
+        <translation>Avbryt</translation>
     </message>
     <message>
         <location filename="translationhandler.cpp" line="31"/>
         <source>Close</source>
-        <translation type="unfinished">Stäng</translation>
+        <translation>Stäng</translation>
     </message>
     <message>
         <location filename="translationhandler.cpp" line="32"/>
         <source>Save</source>
-        <translation type="unfinished"></translation>
+        <translation>Spara</translation>
     </message>
 </context>
 <context>
@@ -1029,13 +1051,13 @@ Options:
         <translation>Okänt språk valt!</translation>
     </message>
     <message>
-        <location filename="translationhandler.cpp" line="103"/>
+        <location filename="translationhandler.cpp" line="105"/>
         <source>Language file %1 not found!</source>
         <oldsource>Language file %1.qm not found!</oldsource>
         <translation>Språk filen %1 hittades ej!</translation>
     </message>
     <message>
-        <location filename="translationhandler.cpp" line="109"/>
+        <location filename="translationhandler.cpp" line="111"/>
         <source>Failed to load translation for language %1 from file %2</source>
         <oldsource>Failed to load translation for language %1 from file %2.qm</oldsource>
         <translation>Misslyckades med att ladda översättningen för %1 från filen %2</translation>
@@ -1072,7 +1094,7 @@ Options:
         <location filename="resultstree.cpp" line="206"/>
         <location filename="resultstree.cpp" line="727"/>
         <source>[Inconclusive]</source>
-        <translation type="unfinished"></translation>
+        <translation>[Inconclusive]</translation>
     </message>
     <message>
         <location filename="resultstree.cpp" line="264"/>
@@ -1097,7 +1119,7 @@ Options:
     <message>
         <location filename="resultstree.cpp" line="536"/>
         <source>Copy message id</source>
-        <translation type="unfinished"></translation>
+        <translation>Kopiera meddelande id</translation>
     </message>
     <message>
         <location filename="resultstree.cpp" line="537"/>
@@ -1107,7 +1129,7 @@ Options:
     <message>
         <location filename="resultstree.cpp" line="538"/>
         <source>Hide all with id</source>
-        <translation type="unfinished"></translation>
+        <translation>Dölj alla med id</translation>
     </message>
     <message>
         <location filename="resultstree.cpp" line="587"/>
@@ -1121,14 +1143,18 @@ Options:
 
 Configure the editor application for Cppcheck in preferences/Applications.</source>
         <oldsource>Configure the text file viewer program in Cppcheck preferences/Applications.</oldsource>
-        <translation type="unfinished">Konfigurera program i inställningar/program.</translation>
+        <translation>Ingen editor konfigurerad.
+
+Konfigurera program i inställningar/program.</translation>
     </message>
     <message>
         <location filename="resultstree.cpp" line="602"/>
         <source>No default editor application selected.
 
 Please select the default editor application in preferences/Applications.</source>
-        <translation type="unfinished"></translation>
+        <translation>Ingen standard editor vald.
+
+Vänligen välj standard editor i inställningar/Program.</translation>
     </message>
     <message>
         <location filename="resultstree.cpp" line="631"/>
@@ -1161,7 +1187,7 @@ Välj mappen där filen finns.</translation>
     <message>
         <location filename="resultstree.cpp" line="1045"/>
         <source>Id</source>
-        <translation type="unfinished"></translation>
+        <translation>Id</translation>
     </message>
     <message>
         <location filename="resultstree.cpp" line="246"/>
@@ -1199,7 +1225,7 @@ Välj mappen där filen finns.</translation>
     <message>
         <location filename="resultsview.cpp" line="201"/>
         <source>%p% (%1 of %2 files checked)</source>
-        <translation type="unfinished"></translation>
+        <translation>%p% (%1 av %2 filer analyserade)</translation>
     </message>
     <message>
         <location filename="resultsview.cpp" line="214"/>
@@ -1239,7 +1265,7 @@ För att ställa in vilka fel som skall visas använd visa menyn.</translation>
     <message>
         <location filename="resultsview.cpp" line="334"/>
         <source>Id</source>
-        <translation type="unfinished"></translation>
+        <translation>Id</translation>
     </message>
     <message>
         <location filename="resultsview.cpp" line="140"/>
@@ -1263,17 +1289,17 @@ För att ställa in vilka fel som skall visas använd visa menyn.</translation>
     <message>
         <location filename="scratchpad.ui" line="14"/>
         <source>Scratchpad</source>
-        <translation type="unfinished"></translation>
+        <translation>Scratchpad</translation>
     </message>
     <message>
         <location filename="scratchpad.ui" line="48"/>
         <source>filename</source>
-        <translation type="unfinished"></translation>
+        <translation>Filnamn</translation>
     </message>
     <message>
         <location filename="scratchpad.ui" line="55"/>
         <source>Check</source>
-        <translation type="unfinished"></translation>
+        <translation>Analysera</translation>
     </message>
 </context>
 <context>
@@ -1328,7 +1354,7 @@ För att ställa in vilka fel som skall visas använd visa menyn.</translation>
     <message>
         <location filename="settings.ui" line="135"/>
         <source>Display error Id in column &quot;Id&quot;</source>
-        <translation type="unfinished"></translation>
+        <translation>Visa meddelande id i kolumn &quot;Id&quot;</translation>
     </message>
     <message>
         <location filename="settings.ui" line="142"/>
@@ -1389,17 +1415,17 @@ För att ställa in vilka fel som skall visas använd visa menyn.</translation>
     <message>
         <location filename="settings.ui" line="329"/>
         <source>Advanced</source>
-        <translation type="unfinished"></translation>
+        <translation>Avancerade</translation>
     </message>
     <message>
         <location filename="settings.ui" line="335"/>
         <source>&amp;Show inconclusive errors</source>
-        <translation type="unfinished"></translation>
+        <translation>Visa inconclusive meddelanden</translation>
     </message>
     <message>
         <location filename="settings.ui" line="342"/>
         <source>S&amp;how internal warnings in log</source>
-        <translation type="unfinished"></translation>
+        <translation>Visa interna fel i loggen</translation>
     </message>
 </context>
 <context>
@@ -1574,67 +1600,67 @@ För att ställa in vilka fel som skall visas använd visa menyn.</translation>
     <message>
         <location filename="statsdialog.cpp" line="96"/>
         <source>Project Settings</source>
-        <translation type="unfinished"></translation>
+        <translation>Projekt inställningar</translation>
     </message>
     <message>
         <location filename="statsdialog.cpp" line="98"/>
         <source>Paths</source>
-        <translation type="unfinished">Sökvägar</translation>
+        <translation>Sökvägar</translation>
     </message>
     <message>
         <location filename="statsdialog.cpp" line="99"/>
         <source>Include paths</source>
-        <translation type="unfinished"></translation>
+        <translation>Include sökvägar</translation>
     </message>
     <message>
         <location filename="statsdialog.cpp" line="100"/>
         <source>Defines</source>
-        <translation type="unfinished"></translation>
+        <translation>Definitioner</translation>
     </message>
     <message>
         <location filename="statsdialog.cpp" line="102"/>
         <source>Path selected</source>
-        <translation type="unfinished"></translation>
+        <translation>Vald sökväg</translation>
     </message>
     <message>
         <location filename="statsdialog.cpp" line="103"/>
         <source>Number of files scanned</source>
-        <translation type="unfinished"></translation>
+        <translation>Antal analyserade filer</translation>
     </message>
     <message>
         <location filename="statsdialog.cpp" line="104"/>
         <source>Scan duration</source>
-        <translation type="unfinished"></translation>
+        <translation>Tid</translation>
     </message>
     <message>
         <location filename="statsdialog.cpp" line="106"/>
         <source>Errors</source>
-        <translation type="unfinished">Fel</translation>
+        <translation>Fel</translation>
     </message>
     <message>
         <location filename="statsdialog.cpp" line="107"/>
         <source>Warnings</source>
-        <translation type="unfinished">Varningar</translation>
+        <translation>Varningar</translation>
     </message>
     <message>
         <location filename="statsdialog.cpp" line="108"/>
         <source>Style warnings</source>
-        <translation type="unfinished">Stil varningar</translation>
+        <translation>Stil varningar</translation>
     </message>
     <message>
         <location filename="statsdialog.cpp" line="109"/>
         <source>Portability warnings</source>
-        <translation type="unfinished"></translation>
+        <translation>Portabilitetsvarningar</translation>
     </message>
     <message>
         <location filename="statsdialog.cpp" line="110"/>
         <source>Performance warnings</source>
-        <translation type="unfinished">Prestanda varningar</translation>
+        <translation>Prestanda varningar</translation>
     </message>
     <message>
         <location filename="statsdialog.cpp" line="111"/>
         <source>Information messages</source>
-        <translation type="unfinished"></translation>
+        <translation>Informationsmeddelanden</translation>
     </message>
 </context>
 <context>
@@ -1642,28 +1668,28 @@ För att ställa in vilka fel som skall visas använd visa menyn.</translation>
     <message>
         <location filename="threadresult.cpp" line="54"/>
         <source>%1 of %2 files checked</source>
-        <translation type="unfinished"></translation>
+        <translation>%1 av %2 filer analyserade</translation>
     </message>
 </context>
 <context>
     <name>TranslationHandler</name>
     <message>
-        <location filename="translationhandler.cpp" line="115"/>
+        <location filename="translationhandler.cpp" line="117"/>
         <source>Failed to change the user interface language:
 
 %1
 
 The user interface language has been reset to English. Open the Preferences-dialog to select any of the available languages.</source>
-        <translation type="unfinished">Misslyckades att ändra språk:
+        <translation>Misslyckades att ändra språk:
 
 %1
 
 Språket har nollställts till Engelska. Öppna Preferences och välj något av de tillgängliga språken.</translation>
     </message>
     <message>
-        <location filename="translationhandler.cpp" line="121"/>
+        <location filename="translationhandler.cpp" line="123"/>
         <source>Cppcheck</source>
-        <translation type="unfinished">Cppcheck</translation>
+        <translation>Cppcheck</translation>
     </message>
 </context>
 <context>
@@ -1671,7 +1697,7 @@ Språket har nollställts till Engelska. Öppna Preferences och välj något av
     <message>
         <location filename="txtreport.cpp" line="73"/>
         <source>inconclusive</source>
-        <translation type="unfinished"></translation>
+        <translation>inconclusive</translation>
     </message>
 </context>
 </TS>
