@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,18 +43,19 @@
 #include "logview.h"
 #include "filelist.h"
 #include "showtypes.h"
-#include "selectfilesdialog.h"
 
 static const QString OnlineHelpURL("http://cppcheck.sourceforge.net/manual.html");
 
-MainWindow::MainWindow() :
-    mSettings(new QSettings("Cppcheck", "Cppcheck-GUI", this)),
+MainWindow::MainWindow(TranslationHandler* th, QSettings* settings) :
+    mSettings(settings),
     mApplications(new ApplicationList(this)),
-    mTranslation(new TranslationHandler(this)),
+    mTranslation(th),
     mLogView(NULL),
     mScratchPad(NULL),
     mProject(NULL),
     mPlatformActions(new QActionGroup(this)),
+    mCStandardActions(new QActionGroup(this)),
+    mCppStandardActions(new QActionGroup(this)),
     mExiting(false)
 {
     mUI.setupUi(this);
@@ -80,7 +81,7 @@ MainWindow::MainWindow() :
     connect(mUI.mActionCheckDirectory, SIGNAL(triggered()), this, SLOT(CheckDirectory()));
     connect(mUI.mActionSettings, SIGNAL(triggered()), this, SLOT(ProgramSettings()));
     connect(mUI.mActionClearResults, SIGNAL(triggered()), this, SLOT(ClearResults()));
-    connect(mUI.mActionOpenXML, SIGNAL(triggered()), this, SLOT(OpenXML()));
+    connect(mUI.mActionOpenXML, SIGNAL(triggered()), this, SLOT(OpenResults()));
 
     connect(mUI.mActionShowStyle, SIGNAL(toggled(bool)), this, SLOT(ShowStyle(bool)));
     connect(mUI.mActionShowErrors, SIGNAL(toggled(bool)), this, SLOT(ShowErrors(bool)));
@@ -170,6 +171,13 @@ MainWindow::MainWindow() :
         connect(act, SIGNAL(triggered()), this, SLOT(SelectPlatform()));
     }
 
+    mUI.mActionC89->setActionGroup(mCStandardActions);
+    mUI.mActionC99->setActionGroup(mCStandardActions);
+    mUI.mActionC11->setActionGroup(mCStandardActions);
+
+    mUI.mActionCpp03->setActionGroup(mCppStandardActions);
+    mUI.mActionCpp11->setActionGroup(mCppStandardActions);
+
     // For Windows platforms default to Win32 checked platform.
     // For other platforms default to unspecified/default which means the
     // platform Cppcheck GUI was compiled on.
@@ -195,6 +203,22 @@ void MainWindow::HandleCLIParams(const QStringList &params)
         const int ind = params.indexOf("-p");
         if ((ind + 1) < params.length())
             LoadProjectFile(params[ind + 1]);
+    } else if (params.contains("-l")) {
+        QString logFile;
+        const int ind = params.indexOf("-l");
+        if ((ind + 1) < params.length())
+            logFile = params[ind + 1];
+
+        if (params.contains("-d")) {
+            QString checkedDir;
+            const int ind = params.indexOf("-d");
+            if ((ind + 1) < params.length())
+                checkedDir = params[ind + 1];
+
+            LoadResults(logFile, checkedDir);
+        } else {
+            LoadResults(logFile);
+        }
     } else
         DoCheckFiles(params);
 }
@@ -217,10 +241,16 @@ void MainWindow::LoadSettings()
     mUI.mActionShowPerformance->setChecked(types->isShown(ShowTypes::ShowPerformance));
     mUI.mActionShowInformation->setChecked(types->isShown(ShowTypes::ShowInformation));
 
-    const bool stdCpp11 = mSettings->value(SETTINGS_STD_CPP11, false).toBool();
-    mUI.mActionCplusplus11->setChecked(stdCpp11);
-    const bool stdC99 = mSettings->value(SETTINGS_STD_C99, false).toBool();
-    mUI.mActionC99->setChecked(stdC99);
+    const bool stdCpp03 = mSettings->value(SETTINGS_STD_CPP03, false).toBool();
+    mUI.mActionCpp03->setChecked(stdCpp03);
+    const bool stdCpp11 = mSettings->value(SETTINGS_STD_CPP11, true).toBool();
+    mUI.mActionCpp11->setChecked(stdCpp11 || !stdCpp03);
+    const bool stdC89 = mSettings->value(SETTINGS_STD_C89, false).toBool();
+    mUI.mActionC89->setChecked(stdC89);
+    const bool stdC11 = mSettings->value(SETTINGS_STD_C11, false).toBool();
+    mUI.mActionC11->setChecked(stdC11);
+    const bool stdC99 = mSettings->value(SETTINGS_STD_C99, true).toBool();
+    mUI.mActionC99->setChecked(stdC99 || (!stdC89 && !stdC11));
     const bool stdPosix = mSettings->value(SETTINGS_STD_POSIX, false).toBool();
     mUI.mActionPosix->setChecked(stdPosix);
 
@@ -237,8 +267,6 @@ void MainWindow::LoadSettings()
     mUI.mActionToolBarFilter->setChecked(showFilterToolbar);
     mUI.mToolBarFilter->setVisible(showFilterToolbar);
 
-    SetLanguage(mSettings->value(SETTINGS_LANGUAGE, mTranslation->SuggestLanguage()).toString());
-
     bool succeeded = mApplications->LoadSettings();
     if (!succeeded) {
         const QString msg = tr("There was a problem with loading the editor application settings.\n\n"
@@ -271,8 +299,11 @@ void MainWindow::SaveSettings() const
     mSettings->setValue(SETTINGS_SHOW_PERFORMANCE, mUI.mActionShowPerformance->isChecked());
     mSettings->setValue(SETTINGS_SHOW_INFORMATION, mUI.mActionShowInformation->isChecked());
 
-    mSettings->setValue(SETTINGS_STD_CPP11, mUI.mActionCplusplus11->isChecked());
+    mSettings->setValue(SETTINGS_STD_CPP03, mUI.mActionCpp03->isChecked());
+    mSettings->setValue(SETTINGS_STD_CPP11, mUI.mActionCpp11->isChecked());
+    mSettings->setValue(SETTINGS_STD_C89, mUI.mActionC89->isChecked());
     mSettings->setValue(SETTINGS_STD_C99, mUI.mActionC99->isChecked());
+    mSettings->setValue(SETTINGS_STD_C11, mUI.mActionC11->isChecked());
     mSettings->setValue(SETTINGS_STD_POSIX, mUI.mActionPosix->isChecked());
 
     // Main window settings
@@ -317,7 +348,7 @@ void MainWindow::DoCheckFiles(const QStringList &files)
     mThread->SetFiles(fileNames);
     QDir inf(mCurrentDirectory);
     const QString checkPath = inf.canonicalPath();
-    mSettings->setValue(SETTINGS_CHECK_PATH, checkPath);
+    SetPath(SETTINGS_LAST_CHECK_PATH, checkPath);
 
     CheckLockDownUI(); // lock UI while checking
 
@@ -374,15 +405,9 @@ QStringList MainWindow::SelectFilesToCheck(QFileDialog::FileMode mode)
     // QFileDialog::getExistingDirectory() because they show native Windows
     // selection dialog which is a lot more usable than Qt:s own dialog.
     if (mode == QFileDialog::ExistingFiles) {
-        SelectFilesDialog dialog(this);
-        if (dialog.exec() == QDialog::Accepted) {
-            selected = dialog.getFiles();
-        }
-        /*
-                selected = QFileDialog::getOpenFileNames(this,
-                           tr("Select files to check"),
-                           mSettings->value(SETTINGS_CHECK_PATH, "").toString());
-        */
+        selected = QFileDialog::getOpenFileNames(this,
+                   tr("Select files to check"),
+                   GetPath(SETTINGS_LAST_CHECK_PATH));
         if (selected.isEmpty())
             mCurrentDirectory.clear();
         else {
@@ -393,7 +418,7 @@ QStringList MainWindow::SelectFilesToCheck(QFileDialog::FileMode mode)
     } else if (mode == QFileDialog::DirectoryOnly) {
         QString dir = QFileDialog::getExistingDirectory(this,
                       tr("Select directory to check"),
-                      mSettings->value(SETTINGS_CHECK_PATH, "").toString());
+                      GetPath(SETTINGS_LAST_CHECK_PATH));
         if (!dir.isEmpty()) {
             qDebug() << "Setting current directory to: " << dir;
             mCurrentDirectory = dir;
@@ -403,6 +428,8 @@ QStringList MainWindow::SelectFilesToCheck(QFileDialog::FileMode mode)
         }
     }
 
+    SetPath(SETTINGS_LAST_CHECK_PATH, mCurrentDirectory);
+
     return selected;
 }
 
@@ -485,6 +512,28 @@ void MainWindow::AddIncludeDirs(const QStringList &includeDirs, Settings &result
     }
 }
 
+bool MainWindow::LoadLibrary(Library *library, QString filename)
+{
+    // Try to load the library from the project folder..
+    if (mProject) {
+        QString path = QFileInfo(mProject->GetProjectFile()->GetFilename()).canonicalPath();
+        if (library->load(NULL, (path+"/"+filename).toLatin1()))
+            return true;
+    }
+
+    // Try to load the library from the application folder..
+    QString path = QFileInfo(QCoreApplication::applicationFilePath()).canonicalPath();
+    if (library->load(NULL, (path+"/"+filename).toLatin1()))
+        return true;
+
+    // Try to load the library from the cfg subfolder..
+    path = path + "/cfg";
+    if (library->load(NULL, (path+"/"+filename).toLatin1()))
+        return true;
+
+    return false;
+}
+
 Settings MainWindow::GetCppcheckSettings()
 {
     Settings result;
@@ -502,6 +551,22 @@ Settings MainWindow::GetCppcheckSettings()
                 result.userDefines += ";";
             result.userDefines += define.toStdString();
         }
+
+        QStringList libraries = pfile->GetLibraries();
+        foreach(QString library, libraries) {
+            const QString filename = library + ".cfg";
+            if (!LoadLibrary(&result.library, filename))
+                QMessageBox::information(this, tr("Information"), tr("Failed to load the selected library %1").arg(filename));
+        }
+
+        QStringList suppressions = pfile->GetSuppressions();
+        foreach(QString suppression, suppressions) {
+            result.nomsg.addSuppressionLine(suppression.toStdString());
+        }
+
+        // Only check the given -D configuration
+        if (!defines.isEmpty())
+            result._maxConfigs = 1;
     }
 
     // Include directories (and files) are searched in listed order.
@@ -513,6 +578,7 @@ Settings MainWindow::GetCppcheckSettings()
         AddIncludeDirs(includes, result);
     }
 
+    result.addEnabled("warning");
     result.addEnabled("style");
     result.addEnabled("performance");
     result.addEnabled("portability");
@@ -528,11 +594,19 @@ Settings MainWindow::GetCppcheckSettings()
     result._inlineSuppressions = mSettings->value(SETTINGS_INLINE_SUPPRESSIONS, false).toBool();
     result.inconclusive = mSettings->value(SETTINGS_INCONCLUSIVE_ERRORS, false).toBool();
     result.platformType = (Settings::PlatformType) mSettings->value(SETTINGS_CHECKED_PLATFORM, 0).toInt();
-    result.standards.cpp = mSettings->value(SETTINGS_STD_CPP11, false).toBool() ? Standards::CPP11 : Standards::CPP03;
-    result.standards.c = mSettings->value(SETTINGS_STD_C99, false).toBool() ? Standards::C99 : Standards::C89;
+    result.standards.cpp = mSettings->value(SETTINGS_STD_CPP11, true).toBool() ? Standards::CPP11 : Standards::CPP03;
+    result.standards.c = mSettings->value(SETTINGS_STD_C99, true).toBool() ? Standards::C99 : (mSettings->value(SETTINGS_STD_C11, false).toBool() ? Standards::C11 : Standards::C89);
     result.standards.posix = mSettings->value(SETTINGS_STD_POSIX, false).toBool();
 
-    if (result._jobs <= 0) {
+    bool std = LoadLibrary(&result.library, "std.cfg");
+    bool posix = true;
+    if (result.standards.posix)
+        posix = LoadLibrary(&result.library, "posix.cfg");
+
+    if (!std || !posix)
+        QMessageBox::warning(this, tr("Error"), tr("Failed to load %1. Your Cppcheck installation is broken.").arg(!std ? "std.cfg" : "posix.cfg"));
+
+    if (result._jobs <= 1) {
         result._jobs = 1;
     }
 
@@ -553,8 +627,8 @@ void MainWindow::CheckDone()
     EnableProjectActions(true);
     EnableProjectOpenActions(true);
     mPlatformActions->setEnabled(true);
-    mUI.mActionCplusplus11->setEnabled(true);
-    mUI.mActionC99->setEnabled(true);
+    mCStandardActions->setEnabled(true);
+    mCppStandardActions->setEnabled(true);
     mUI.mActionPosix->setEnabled(true);
     if (mScratchPad)
         mScratchPad->setEnabled(true);
@@ -581,8 +655,8 @@ void MainWindow::CheckLockDownUI()
     EnableProjectActions(false);
     EnableProjectOpenActions(false);
     mPlatformActions->setEnabled(false);
-    mUI.mActionCplusplus11->setEnabled(false);
-    mUI.mActionC99->setEnabled(false);
+    mCStandardActions->setEnabled(false);
+    mCppStandardActions->setEnabled(false);
     mUI.mActionPosix->setEnabled(false);
     if (mScratchPad)
         mScratchPad->setEnabled(false);
@@ -637,7 +711,7 @@ void MainWindow::ClearResults()
     mUI.mActionSave->setEnabled(false);
 }
 
-void MainWindow::OpenXML()
+void MainWindow::OpenResults()
 {
     if (mUI.mResults->HasResults()) {
         QMessageBox msgBox(this);
@@ -660,16 +734,30 @@ void MainWindow::OpenXML()
     const QString filter(tr("XML files (*.xml)"));
     QString selectedFile = QFileDialog::getOpenFileName(this,
                            tr("Open the report file"),
-                           QString(),
+                           GetPath(SETTINGS_LAST_RESULT_PATH),
                            filter,
                            &selectedFilter);
 
+    if (!selectedFile.isEmpty()) {
+        LoadResults(selectedFile);
+    }
+}
+
+void MainWindow::LoadResults(const QString selectedFile)
+{
     if (!selectedFile.isEmpty()) {
         mUI.mResults->Clear(true);
         mUI.mResults->ReadErrorsXml(selectedFile);
+        SetPath(SETTINGS_LAST_RESULT_PATH, selectedFile);
     }
 }
 
+void MainWindow::LoadResults(const QString selectedFile, const QString sourceDirectory)
+{
+    LoadResults(selectedFile);
+    mUI.mResults->SetCheckDirectory(sourceDirectory);
+}
+
 void MainWindow::EnableCheckButtons(bool enable)
 {
     mUI.mActionStop->setEnabled(!enable);
@@ -792,7 +880,7 @@ void MainWindow::Save()
     const QString filter(tr("XML files version 2 (*.xml);;XML files version 1 (*.xml);;Text files (*.txt);;CSV files (*.csv)"));
     QString selectedFile = QFileDialog::getSaveFileName(this,
                            tr("Save the report file"),
-                           QString(),
+                           GetPath(SETTINGS_LAST_RESULT_PATH),
                            filter,
                            &selectedFilter);
 
@@ -824,6 +912,7 @@ void MainWindow::Save()
         }
 
         mUI.mResults->Save(selectedFile, type);
+        SetPath(SETTINGS_LAST_RESULT_PATH, selectedFile);
     }
 }
 
@@ -863,23 +952,12 @@ void MainWindow::SetLanguage(const QString &code)
     if (currentLang == code)
         return;
 
-    QString error;
-    if (!mTranslation->SetLanguage(code, error)) {
-        const QString msg(tr("Failed to change the user interface language:"
-                             "\n\n%1\n\n"
-                             "The user interface language has been reset to English. Open "
-                             "the Preferences-dialog to select any of the available "
-                             "languages.").arg(error));
-        QMessageBox msgBox(QMessageBox::Warning,
-                           tr("Cppcheck"),
-                           msg,
-                           QMessageBox::Ok,
-                           this);
-        msgBox.exec();
-    } else {
+    if (mTranslation->SetLanguage(code)) {
         //Translate everything that is visible here
         mUI.retranslateUi(this);
         mUI.mResults->Translate();
+        delete mLogView;
+        mLogView = 0;
     }
 }
 
@@ -912,13 +990,13 @@ void MainWindow::OpenProjectFile()
     const QString filter = tr("Project files (*.cppcheck);;All files(*.*)");
     const QString filepath = QFileDialog::getOpenFileName(this,
                              tr("Select Project File"),
-                             lastPath,
+                             GetPath(SETTINGS_LAST_PROJECT_PATH),
                              filter);
 
     if (!filepath.isEmpty()) {
         const QFileInfo fi(filepath);
         if (fi.exists() && fi.isFile() && fi.isReadable()) {
-            mSettings->setValue(SETTINGS_LAST_PROJECT_PATH, fi.path());
+            SetPath(SETTINGS_LAST_PROJECT_PATH, filepath);
             LoadProjectFile(filepath);
         }
     }
@@ -995,12 +1073,14 @@ void MainWindow::NewProjectFile()
     const QString filter = tr("Project files (*.cppcheck);;All files(*.*)");
     QString filepath = QFileDialog::getSaveFileName(this,
                        tr("Select Project Filename"),
-                       QString(),
+                       GetPath(SETTINGS_LAST_PROJECT_PATH),
                        filter);
 
     if (filepath.isEmpty())
         return;
 
+    SetPath(SETTINGS_LAST_PROJECT_PATH, filepath);
+
     EnableProjectActions(true);
     QFileInfo inf(filepath);
     const QString filename = inf.fileName();
