@@ -52,7 +52,12 @@ void CheckAssignIf::assignIf()
                 bitop = tok->strAt(2).at(0);
                 num = MathLib::toLongNumber(tok->next()->str());
             } else {
-                const Token * const endToken = Token::findsimplematch(tok, ";");
+                const Token *endToken = Token::findsimplematch(tok, ";");
+
+                // Casting address
+                if (Token::Match(endToken->tokAt(-4), "* ) & %any% ;"))
+                    endToken = NULL;
+
                 if (endToken && Token::Match(endToken->tokAt(-2), "[&|] %num% ;")) {
                     bitop = endToken->strAt(-2).at(0);
                     num = MathLib::toLongNumber(endToken->previous()->str());
@@ -65,7 +70,7 @@ void CheckAssignIf::assignIf()
             if (num < 0 && bitop == '|')
                 continue;
 
-            assignIfParseScope(tok, tok->tokAt(4), var->varId(), var->isLocal(), bitop, num);
+            assignIfParseScope(tok, tok->tokAt(4), var->declarationId(), var->isLocal(), bitop, num);
         }
     }
 }
@@ -78,6 +83,8 @@ bool CheckAssignIf::assignIfParseScope(const Token * const assignTok,
                                        const char bitop,
                                        const MathLib::bigint num)
 {
+    bool ret = false;
+
     for (const Token *tok2 = startTok; tok2; tok2 = tok2->next()) {
         if (Token::Match(tok2->tokAt(2), "%varid% %cop% %num% ;", varid) && tok2->strAt(3) == std::string(1U, bitop)) {
             const MathLib::bigint num2 = MathLib::toLongNumber(tok2->strAt(4));
@@ -92,10 +99,28 @@ bool CheckAssignIf::assignIfParseScope(const Token * const assignTok,
             }
             return true;
         }
-        if (Token::Match(tok2, "[(,] &| %varid% [,)]", varid))
-            return true;
+        if (Token::Match(tok2, "[(,] &| %varid% [,)]", varid)) {
+            unsigned int argumentNumber = 0;
+            const Token *ftok;
+            for (ftok = tok2; ftok && ftok->str() != "("; ftok = ftok->previous()) {
+                if (ftok->str() == ")")
+                    ftok = ftok->link();
+                else if (ftok->str() == ",")
+                    argumentNumber++;
+            }
+            ftok = ftok ? ftok->previous() : NULL;
+            if (!(ftok && ftok->function()))
+                return true;
+            const Variable *par = ftok->function()->getArgumentVar(argumentNumber);
+            if (par == NULL || par->isReference() || par->isPointer())
+                return true;
+        }
         if (tok2->str() == "}")
             return false;
+        if (Token::Match(tok2, "break|continue|return"))
+            ret = true;
+        if (ret && tok2->str() == ";")
+            return false;
         if (!islocal && Token::Match(tok2, "%var% (") && !Token::simpleMatch(tok2->next()->link(), ") {"))
             return true;
         if (Token::Match(tok2, "if|while (")) {
@@ -161,33 +186,50 @@ void CheckAssignIf::mismatchingBitAndError(const Token *tok1, const MathLib::big
 }
 
 
+static void getnumchildren(const Token *tok, std::list<MathLib::bigint> &numchildren)
+{
+    if (tok->astOperand1() && tok->astOperand1()->isNumber())
+        numchildren.push_back(MathLib::toLongNumber(tok->astOperand1()->str()));
+    else if (tok->astOperand1() && tok->str() == tok->astOperand1()->str())
+        getnumchildren(tok->astOperand1(), numchildren);
+    if (tok->astOperand2() && tok->astOperand2()->isNumber())
+        numchildren.push_back(MathLib::toLongNumber(tok->astOperand2()->str()));
+    else if (tok->astOperand2() && tok->str() == tok->astOperand2()->str())
+        getnumchildren(tok->astOperand2(), numchildren);
+}
 
 void CheckAssignIf::comparison()
 {
     if (!_settings->isEnabled("style"))
         return;
 
+    // Experimental code based on AST
     for (const Token *tok = _tokenizer->tokens(); tok; tok = tok->next()) {
-        if (Token::Match(tok, "&|%or% %num% )| ==|!= %num% &&|%oror%|)")) {
-            const MathLib::bigint num1 = MathLib::toLongNumber(tok->strAt(1));
-            if (num1 < 0)
+        if (Token::Match(tok, "==|!=")) {
+            const Token *expr1 = tok->astOperand1();
+            const Token *expr2 = tok->astOperand2();
+            if (!expr1 || !expr2)
                 continue;
-
-            const Token *compareToken = tok->tokAt(2);
-            if (compareToken->str() == ")") {
-                if (!Token::Match(compareToken->link()->previous(), "(|%oror%|&&"))
-                    continue;
-                compareToken = compareToken->next();
-            }
-
-            const MathLib::bigint num2 = MathLib::toLongNumber(compareToken->strAt(1));
+            if (expr1->isNumber())
+                std::swap(expr1,expr2);
+            if (!expr2->isNumber())
+                continue;
+            const MathLib::bigint num2 = MathLib::toLongNumber(expr2->str());
             if (num2 < 0)
                 continue;
-
-            if ((tok->str() == "&" && (num1 & num2) != num2) ||
-                (tok->str() == "|" && (num1 | num2) != num2)) {
-                const std::string& op(compareToken->str());
-                comparisonError(tok, tok->str(), num1, op, num2, op=="==" ? false : true);
+            if (!Token::Match(expr1,"[&|]"))
+                continue;
+            std::list<MathLib::bigint> numbers;
+            getnumchildren(expr1, numbers);
+            for (std::list<MathLib::bigint>::const_iterator num = numbers.begin(); num != numbers.end(); ++num) {
+                const MathLib::bigint num1 = *num;
+                if (num1 < 0)
+                    continue;
+                if ((expr1->str() == "&" && (num1 & num2) != num2) ||
+                    (expr1->str() == "|" && (num1 | num2) != num2)) {
+                    const std::string& op(tok->str());
+                    comparisonError(expr1, expr1->str(), num1, op, num2, op=="==" ? false : true);
+                }
             }
         }
     }
