@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -73,6 +73,8 @@ private:
 
     void assignIfError(const Token *tok1, const Token *tok2, const std::string &condition, bool result);
 
+    void mismatchingBitAndError(const Token *tok1, const MathLib::bigint num1, const Token *tok2, const MathLib::bigint num2);
+
     void comparisonError(const Token *tok,
                          const std::string &bitop,
                          MathLib::bigint value1,
@@ -87,19 +89,21 @@ private:
         c.assignIfError(0, 0, "", false);
         c.comparisonError(0, "&", 6, "==", 1, false);
         c.multiConditionError(0,1);
+        c.mismatchingBitAndError(0,0xf0, 0, 1);
     }
 
     static std::string myName() {
-        return "Match assignments and conditions";
+        return "AssignIf";
     }
 
     std::string classInfo() const {
         return "Match assignments and conditions:\n"
                "* Mismatching assignment and comparison => comparison is always true/false\n"
                "* Mismatching lhs and rhs in comparison => comparison is always true/false\n"
-               "* Detect matching 'if' and 'else if' conditions\n";
+               "* Detect matching 'if' and 'else if' conditions\n"
+               "* Mismatching bitand (a &= 0xf0; a &= 1; => a = 0)\n";
     }
 };
 /// @}
 //---------------------------------------------------------------------------
-#endif
+#endif // checkassignifH
