@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
 
 
 //---------------------------------------------------------------------------
-#ifndef CheckAutoVariablesH
-#define CheckAutoVariablesH
+#ifndef checkautovariablesH
+#define checkautovariablesH
 //---------------------------------------------------------------------------
 
 #include "config.h"
@@ -27,23 +27,25 @@
 #include "token.h"
 
 /// @addtogroup Checks
+/** @brief Various small checks for automatic variables */
 /// @{
 
 
 class CPPCHECKLIB CheckAutoVariables : public Check {
 public:
     /** This constructor is used when registering the CheckClass */
-    CheckAutoVariables() : Check(myName())
-    { }
+    CheckAutoVariables() : Check(myName()) {
+    }
 
     /** This constructor is used when running checks. */
     CheckAutoVariables(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger)
-        : Check(myName(), tokenizer, settings, errorLogger)
-    { }
+        : Check(myName(), tokenizer, settings, errorLogger) {
+    }
 
     /** @brief Run checks against the normal token list */
     void runChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) {
         CheckAutoVariables checkAutoVariables(tokenizer, settings, errorLogger);
+        checkAutoVariables.assignFunctionArg();
         checkAutoVariables.returnReference();
     }
 
@@ -53,6 +55,9 @@ public:
         checkAutoVariables.returnPointerToLocalArray();
     }
 
+    /** assign function argument */
+    void assignFunctionArg();
+
     /** Check auto variables */
     void autoVariables();
 
@@ -63,18 +68,18 @@ public:
     void returnReference();
 
 private:
-    bool isRefPtrArg(unsigned int varId);
-    bool isPtrArg(unsigned int varId);
-    bool isAutoVar(unsigned int varId);
-    bool isAutoVarArray(unsigned int varId);
+    static bool isPtrArg(const Token *tok);
+    static bool isRefPtrArg(const Token *tok);
+    static bool isNonReferenceArg(const Token *tok);
+    static bool isAutoVar(const Token *tok);
+    static bool isAutoVarArray(const Token *tok);
 
     /**
      * Returning a temporary object?
      * @param tok pointing at the "return" token
-     * @param startScope indicates the function scope to be checked
      * @return true if a temporary object is returned
      */
-    bool returnTemporary(const Token *tok, const Scope *startScope) const;
+    bool returnTemporary(const Token *tok) const;
 
     void errorReturnAddressToAutoVariable(const Token *tok);
     void errorReturnPointerToLocalArray(const Token *tok);
@@ -83,6 +88,7 @@ private:
     void errorReturnTempReference(const Token *tok);
     void errorInvalidDeallocation(const Token *tok);
     void errorReturnAddressOfFunctionParameter(const Token *tok, const std::string &varname);
+    void errorUselessAssignmentPtrArg(const Token *tok);
 
     void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) const {
         CheckAutoVariables c(0,settings,errorLogger);
@@ -93,6 +99,7 @@ private:
         c.errorReturnTempReference(0);
         c.errorInvalidDeallocation(0);
         c.errorReturnAddressOfFunctionParameter(0, "parameter");
+        c.errorUselessAssignmentPtrArg(0);
     }
 
     static std::string myName() {
@@ -105,9 +112,10 @@ private:
                "* returning a pointer to auto or temporary variable\n"
                "* assigning address of an variable to an effective parameter of a function\n"
                "* returning reference to local/temporary variable\n"
-               "* returning address of function parameter\n";
+               "* returning address of function parameter\n"
+               "* useless assignment of pointer parameter\n";
     }
 };
 /// @}
 //---------------------------------------------------------------------------
-#endif
+#endif // checkautovariablesH
