@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  */
 
 //---------------------------------------------------------------------------
-#ifndef CheckClassH
-#define CheckClassH
+#ifndef checkclassH
+#define checkclassH
 //---------------------------------------------------------------------------
 
 #include "config.h"
@@ -36,8 +36,8 @@ class Function;
 class CPPCHECKLIB CheckClass : public Check {
 public:
     /** @brief This constructor is used when registering the CheckClass */
-    CheckClass() : Check(myName()), symbolDatabase(NULL)
-    { }
+    CheckClass() : Check(myName()), symbolDatabase(NULL) {
+    }
 
     /** @brief This constructor is used when running checks. */
     CheckClass(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger);
@@ -50,7 +50,7 @@ public:
         CheckClass checkClass(tokenizer, settings, errorLogger);
 
         // can't be a simplified check .. the 'sizeof' is used.
-        checkClass.noMemset();
+        checkClass.checkMemset();
     }
 
     /** @brief Run checks on the simplified token list */
@@ -73,6 +73,9 @@ public:
         checkClass.virtualDestructor();
         checkClass.checkConst();
         checkClass.copyconstructors();
+        checkClass.checkPureVirtualFunctionCall();
+
+        checkClass.checkDuplInheritedMembers();
     }
 
 
@@ -90,8 +93,8 @@ public:
      * It can also overwrite the virtual table.
      * Important: The checking doesn't work on simplified tokens list.
      */
-    void noMemset();
-    void checkMemsetType(const Scope *start, const Token *tok, const Scope *type);
+    void checkMemset();
+    void checkMemsetType(const Scope *start, const Token *tok, const Scope *type, bool allocation);
 
     /** @brief 'operator=' should return something and it should not be const. */
     void operatorEq();
@@ -118,6 +121,12 @@ public:
 
     void copyconstructors();
 
+    /** @brief call of pure virtual funcion */
+    void checkPureVirtualFunctionCall();
+
+    /** @brief Check duplicated inherited members */
+    void checkDuplInheritedMembers();
+
 private:
     const SymbolDatabase *symbolDatabase;
 
@@ -130,6 +139,8 @@ private:
     void operatorEqVarError(const Token *tok, const std::string &classname, const std::string &varname, bool inconclusive);
     void unusedPrivateFunctionError(const Token *tok, const std::string &classname, const std::string &funcname);
     void memsetError(const Token *tok, const std::string &memfunc, const std::string &classname, const std::string &type);
+    void mallocOnClassError(const Token* tok, const std::string &memfunc, const Token* classTok, const std::string &classname);
+    void mallocOnClassWarning(const Token* tok, const std::string &memfunc, const Token* classTok);
     void operatorEqReturnError(const Token *tok, const std::string &className);
     void virtualDestructorError(const Token *tok, const std::string &Base, const std::string &Derived);
     void thisSubtractionError(const Token *tok);
@@ -139,6 +150,8 @@ private:
     void checkConstError2(const Token *tok1, const Token *tok2, const std::string &classname, const std::string &funcname, bool suggestStatic);
     void initializerListError(const Token *tok1,const Token *tok2, const std::string & classname, const std::string &varname);
     void suggestInitializationList(const Token *tok, const std::string& varname);
+    void callsPureVirtualFunctionError(const Function & scopeFunction, const std::list<const Token *> & tokStack, const std::string &purefuncname);
+    void duplInheritedMembersError(const Token* tok1, const Token* tok2, const std::string &derivedname, const std::string &basename, const std::string &variablename, bool derivedIsStruct, bool baseIsStruct);
 
     void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) const {
         CheckClass c(0, settings, errorLogger);
@@ -150,6 +163,8 @@ private:
         c.operatorEqVarError(0, "classname", "", false);
         c.unusedPrivateFunctionError(0, "classname", "funcname");
         c.memsetError(0, "memfunc", "classname", "class");
+        c.mallocOnClassWarning(0, "malloc", 0);
+        c.mallocOnClassError(0, "malloc", 0, "std::string");
         c.operatorEqReturnError(0, "class");
         c.virtualDestructorError(0, "Base", "Derived");
         c.thisSubtractionError(0);
@@ -159,6 +174,7 @@ private:
         c.checkConstError(0, "class", "function", true);
         c.initializerListError(0, 0, "class", "variable");
         c.suggestInitializationList(0, "variable");
+        c.duplInheritedMembersError(0, 0, "class", "class", "variable", false, false);
     }
 
     static std::string myName() {
@@ -172,6 +188,7 @@ private:
                "* Are all variables initialized by the constructors?\n"
                "* Are all variables assigned by 'operator='?\n"
                "* Warn if memset, memcpy etc are used on a class\n"
+               "* Warn if memory for classes is allocated with malloc()\n"
                "* If it's a base class, check that the destructor is virtual\n"
                "* Are there unused private functions?\n"
                "* 'operator=' should return reference to self\n"
@@ -179,7 +196,9 @@ private:
                "* Constness for member functions\n"
                "* Order of initializations\n"
                "* Suggest usage of initialization list\n"
-               "* Suspicious subtraction from 'this'\n";
+               "* Suspicious subtraction from 'this'\n"
+               "* Call of pure virtual function in constructor/destructor\n"
+               "* Duplicated inherited data members\n";
     }
 
     // operatorEqRetRefThis helper function
@@ -244,10 +263,33 @@ private:
      * @param scope pointer to variable Scope
      * @param usage reference to usage vector
      */
-    void initializeVarList(const Function &func, std::list<std::string> &callstack, const Scope *scope, std::vector<Usage> &usage);
+    void initializeVarList(const Function &func, std::list<const Function *> &callstack, const Scope *scope, std::vector<Usage> &usage);
+
+    /**
+     * @brief gives a list of tokens where pure virtual functions are called directly or indirectly
+     * @param function function to be checked
+     * @param callsPureVirtualFunctionMap map of results for already checked functions
+     * @return list of tokens where pure virtual functions are called
+     */
+    const std::list<const Token *> & callsPureVirtualFunction(
+        const Function & function,
+        std::map<const Function *, std::list<const Token *> > & callsPureVirtualFunctionMap);
+
+    /**
+     * @brief looks for the first pure virtual function call stack
+     * @param callsPureVirtualFunctionMap map of results obtained from callsPureVirtualFunction
+     * @param pureCall token where pure virtual function is called directly or indirectly
+     * @param[in,out] pureFuncStack list to append the stack
+     */
+    void getFirstPureVirtualFunctionCallStack(
+        std::map<const Function *, std::list<const Token *> > & callsPureVirtualFunctionMap,
+        const Token & pureCall,
+        std::list<const Token *> & pureFuncStack);
 
     static bool canNotCopy(const Scope *scope);
+
+    static bool canNotMove(const Scope *scope);
 };
 /// @}
 //---------------------------------------------------------------------------
-#endif
+#endif // checkclassH
