@@ -36,15 +36,16 @@ namespace {
  * @brief parse a function call and extract information about variable usage
  * @param tok first token
  * @param var variables that the function read / write.
+ * @param library --library files data
  * @param value 0 => invalid with null pointers as parameter.
- *              1-.. => invalid with uninitialized data.
+ *              1-.. => only invalid with uninitialized data.
  */
-void CheckNullPointer::parseFunctionCall(const Token &tok, std::list<const Token *> &var, unsigned char value)
+void CheckNullPointer::parseFunctionCall(const Token &tok, std::list<const Token *> &var, const Library *library, unsigned char value)
 {
     // standard functions that dereference first parameter..
-    static std::set<std::string> functionNames1_all;
-    static std::set<std::string> functionNames1_nullptr;
-    static std::set<std::string> functionNames1_uninit;
+    static std::set<std::string> functionNames1_all;     // used no matter what 'value' is
+    static std::set<std::string> functionNames1_nullptr; // used only when 'value' is 0
+    static std::set<std::string> functionNames1_uninit;  // used only when 'value' is non-zero
     if (functionNames1_all.empty()) {
         // cstdlib
         functionNames1_all.insert("atoi");
@@ -109,7 +110,6 @@ void CheckNullPointer::parseFunctionCall(const Token &tok, std::list<const Token
         functionNames1_all.insert("fgetwc");
         functionNames1_all.insert("fgetpos");
         functionNames1_all.insert("fsetpos");
-        functionNames1_all.insert("freopen");
         functionNames1_all.insert("fscanf");
         functionNames1_all.insert("fprintf");
         functionNames1_all.insert("fwscanf");
@@ -136,7 +136,6 @@ void CheckNullPointer::parseFunctionCall(const Token &tok, std::list<const Token
         functionNames1_all.insert("ctime");
         functionNames1_all.insert("mktime");
 
-        functionNames1_nullptr.insert("itoa");
         functionNames1_nullptr.insert("memcpy");
         functionNames1_nullptr.insert("memmove");
         functionNames1_nullptr.insert("memset");
@@ -158,13 +157,15 @@ void CheckNullPointer::parseFunctionCall(const Token &tok, std::list<const Token
         functionNames1_nullptr.insert("localtime");
         functionNames1_nullptr.insert("strftime");
 
+        functionNames1_uninit.insert("itoa"); // value to convert
         functionNames1_uninit.insert("perror");
         functionNames1_uninit.insert("fflush");
+        functionNames1_uninit.insert("freopen");
     }
 
     // standard functions that dereference second parameter..
-    static std::set<std::string> functionNames2_all;
-    static std::set<std::string> functionNames2_nullptr;
+    static std::set<std::string> functionNames2_all;     // used no matter what 'value' is
+    static std::set<std::string> functionNames2_nullptr; // used only if 'value' is 0
     if (functionNames2_all.empty()) {
         functionNames2_all.insert("mbstowcs");
         functionNames2_all.insert("wcstombs");
@@ -214,6 +215,7 @@ void CheckNullPointer::parseFunctionCall(const Token &tok, std::list<const Token
         functionNames2_all.insert("putwc");
         functionNames2_all.insert("freopen");
 
+        functionNames2_nullptr.insert("itoa"); // destination buffer
         functionNames2_nullptr.insert("frexp");
         functionNames2_nullptr.insert("modf");
         functionNames2_nullptr.insert("fgetpos");
@@ -227,7 +229,7 @@ void CheckNullPointer::parseFunctionCall(const Token &tok, std::list<const Token
 
     // 1st parameter..
     if ((Token::Match(firstParam, "%var% ,|)") && firstParam->varId() > 0) ||
-        (value == 0 && Token::Match(firstParam, "0 ,|)"))) {
+        (value == 0 && Token::Match(firstParam, "0|NULL ,|)"))) {
         if (functionNames1_all.find(tok.str()) != functionNames1_all.end())
             var.push_back(firstParam);
         else if (value == 0 && functionNames1_nullptr.find(tok.str()) != functionNames1_nullptr.end())
@@ -236,14 +238,22 @@ void CheckNullPointer::parseFunctionCall(const Token &tok, std::list<const Token
             var.push_back(firstParam);
         else if (value == 0 && Token::Match(&tok, "snprintf|vsnprintf|fnprintf|vfnprintf") && secondParam && secondParam->str() != "0") // Only if length (second parameter) is not zero
             var.push_back(firstParam);
+        else if (value == 0 && library != NULL && library->isnullargbad(tok.str(),1))
+            var.push_back(firstParam);
+        else if (value == 1 && library != NULL && library->isuninitargbad(tok.str(),1))
+            var.push_back(firstParam);
     }
 
     // 2nd parameter..
-    if (secondParam && ((value == 0 && secondParam->str() == "0") || (secondParam->varId() > 0))) {
+    if ((value == 0 && Token::Match(secondParam, "0|NULL ,|)")) || (secondParam && secondParam->varId() > 0 && Token::Match(secondParam->next(),"[,)]"))) {
         if (functionNames2_all.find(tok.str()) != functionNames2_all.end())
             var.push_back(secondParam);
         else if (value == 0 && functionNames2_nullptr.find(tok.str()) != functionNames2_nullptr.end())
             var.push_back(secondParam);
+        else if (value == 0 && library != NULL && library->isnullargbad(tok.str(),2))
+            var.push_back(secondParam);
+        else if (value == 1 && library != NULL && library->isuninitargbad(tok.str(),2))
+            var.push_back(secondParam);
     }
 
     if (Token::Match(&tok, "printf|sprintf|snprintf|fprintf|fnprintf|scanf|sscanf|fscanf|wprintf|swprintf|fwprintf|wscanf|swscanf|fwscanf")) {
@@ -292,7 +302,7 @@ void CheckNullPointer::parseFunctionCall(const Token &tok, std::list<const Token
                         continue;
 
                     if ((*i == 'n' || *i == 's' || scan) && (!scan || value == 0)) {
-                        if ((value == 0 && argListTok->str() == "0") || (argListTok->varId() > 0)) {
+                        if ((value == 0 && argListTok->str() == "0") || (argListTok->varId() > 0 && Token::Match(argListTok,"%var% [,)]"))) {
                             var.push_back(argListTok);
                         }
                     }
@@ -323,19 +333,29 @@ bool CheckNullPointer::isPointerDeRef(const Token *tok, bool &unknown)
 
     unknown = false;
 
+    const Token* prev = tok->previous();
+    while (prev->str() == "." || prev->str() == "::") { // Skip over previous member dereferences
+        prev = prev->previous();
+        while (prev->link() && (prev->str() == "]" || prev->str() == ">"))
+            prev = prev->link()->previous();
+        prev = prev->previous();
+    }
+    while (prev->str() == ")") // Skip over casts
+        prev = prev->link()->previous();
+
     // Dereferencing pointer..
-    if (tok->strAt(-1) == "*" && (Token::Match(tok->tokAt(-2), "return|throw|;|{|}|:|[|(|,") || tok->tokAt(-2)->isOp()) && !Token::Match(tok->tokAt(-3), "sizeof|decltype"))
+    if (prev->str() == "*" && (Token::Match(prev->previous(), "return|throw|;|{|}|:|[|(|,") || prev->previous()->isOp()) && !Token::Match(prev->tokAt(-2), "sizeof|decltype|typeof"))
         return true;
 
     // read/write member variable
-    if (!Token::simpleMatch(tok->tokAt(-2), "& (") && !Token::Match(tok->tokAt(-2), "sizeof|decltype (") && tok->strAt(-1) != "&" && Token::Match(tok->next(), ". %var%")) {
+    if (!Token::simpleMatch(prev->previous(), "& (") && !Token::Match(prev->previous(), "sizeof|decltype|typeof (") && prev->str() != "&" && Token::Match(tok->next(), ". %var%")) {
         if (tok->strAt(3) != "(")
             return true;
         unknown = true;
         return false;
     }
 
-    if (Token::Match(tok, "%var% [") && (tok->previous()->str() != "&" || Token::Match(tok->next()->link()->next(), "[.(]")))
+    if (Token::Match(tok, "%var% [") && (prev->str() != "&" || Token::Match(tok->next()->link()->next(), "[.(]")))
         return true;
 
     if (Token::Match(tok, "%var% ("))
@@ -347,19 +367,19 @@ bool CheckNullPointer::isPointerDeRef(const Token *tok, bool &unknown)
         return true;
 
     // std::string dereferences nullpointers
-    if (Token::Match(tok->tokAt(-4), "std :: string|wstring ( %var% )"))
+    if (Token::Match(prev->tokAt(-3), "std :: string|wstring (") && tok->strAt(1) == ")")
         return true;
-    if (Token::Match(tok->tokAt(-2), "%var% ( %var% )")) {
+    if (Token::Match(prev->previous(), "%var% (") && tok->strAt(1) == ")") {
         const Variable* var = tok->tokAt(-2)->variable();
         if (var && !var->isPointer() && !var->isArray() && Token::Match(var->typeStartToken(), "std :: string|wstring !!::"))
             return true;
     }
 
     // streams dereference nullpointers
-    if (Token::Match(tok->previous(), "<<|>> %var%")) {
+    if (Token::Match(prev, "<<|>>")) {
         const Variable* var = tok->variable();
         if (var && var->isPointer() && Token::Match(var->typeStartToken(), "char|wchar_t")) { // Only outputting or reading to char* can cause problems
-            const Token* tok2 = tok->previous(); // Find start of statement
+            const Token* tok2 = prev; // Find start of statement
             for (; tok2; tok2 = tok2->previous()) {
                 if (Token::Match(tok2->previous(), ";|{|}|:"))
                     break;
@@ -377,9 +397,9 @@ bool CheckNullPointer::isPointerDeRef(const Token *tok, bool &unknown)
     const Variable *ovar = NULL;
     if (Token::Match(tok, "%var% ==|!= %var%"))
         ovar = tok->tokAt(2)->variable();
-    else if (Token::Match(tok->tokAt(-2), "%var% ==|!= %var%"))
+    else if (Token::Match(prev->previous(), "%var% ==|!="))
         ovar = tok->tokAt(-2)->variable();
-    else if (Token::Match(tok->tokAt(-2), "%var% =|+ %var% )|]|,|;|+"))
+    else if (Token::Match(prev->previous(), "%var% =|+") && Token::Match(tok->next(), ")|]|,|;|+"))
         ovar = tok->tokAt(-2)->variable();
     if (ovar && !ovar->isPointer() && !ovar->isArray() && Token::Match(ovar->typeStartToken(), "std :: string|wstring !!::"))
         return true;
@@ -392,41 +412,41 @@ bool CheckNullPointer::isPointerDeRef(const Token *tok, bool &unknown)
             return false;
 
         // OK to delete a null
-        if (Token::Match(tok->previous(), "delete %var%") || Token::Match(tok->tokAt(-3), "delete [ ] %var%"))
+        if (Token::Match(prev, "delete") || Token::Match(prev->tokAt(-2), "delete [ ]"))
             return false;
 
         // OK to check if pointer is null
         // OK to take address of pointer
-        if (Token::Match(tok->previous(), "!|& %var%"))
+        if (Token::Match(prev, "!|&"))
             return false;
 
         // OK to check pointer in "= p ? : "
         if (tok->next()->str() == "?" &&
-            (Token::Match(tok->previous(), "return|throw|;|{|}|:|[|(|,") || tok->previous()->isAssignmentOp()))
+            (Token::Match(prev, "return|throw|;|{|}|:|[|(|,") || prev->isAssignmentOp()))
             return false;
 
         // OK to pass pointer to function
-        if (Token::Match(tok->previous(), "[(,] %var% [,)]") &&
-            (!Token::Match(tok->previous(), "( %var%") ||
-             Token::Match(tok->tokAt(-2), "%var% ( %var%")))
+        if (Token::Match(prev, "[(,]") && Token::Match(tok->next(), "[,)]") &&
+            (prev->str() != "(" ||
+             Token::Match(prev->previous(), "%var% (")))
             return false;
 
         // Compare pointer
-        if (Token::Match(tok->previous(), "(|&&|%oror%|==|!= %var%"))
+        if (Token::Match(prev, "(|&&|%oror%|==|!="))
             return false;
         if (Token::Match(tok, "%var% &&|%oror%|==|!=|)"))
             return false;
 
         // Taking address
-        if (Token::Match(tok->previous(), "return|= %var% ;"))
+        if (Token::Match(prev, "return|=") && tok->strAt(1) == ";")
             return false;
 
         // (void)var
-        if (Token::Match(tok->previous(), "[{;}] %var% ;"))
+        if (Token::Match(prev, "[{;}]") && tok->strAt(1) == ";")
             return false;
 
         // Shift pointer (e.g. to cout, but its no char* (see above))
-        if (Token::Match(tok->previous(), "<<|>> %var%"))
+        if (Token::Match(prev, "<<|>>"))
             return false;
 
         // unknown if it's a dereference
@@ -500,6 +520,10 @@ void CheckNullPointer::nullPointerLinkedList()
                 // Variable id for dereferenced variable
                 const unsigned int varid(tok2->varId());
 
+                // We don't support variables without a varid
+                if (varid == 0)
+                    continue;
+
                 if (Token::Match(tok2->tokAt(-2), "%varid% ?", varid))
                     continue;
 
@@ -556,8 +580,8 @@ void CheckNullPointer::nullPointerStructByDeRefAndChec()
                     tok1 = tok1->next();
                 skipvar.insert(tok1->varId());
                 continue;
-            } else if (Token::Match(tok1, "( ! %var% %oror%") ||
-                       Token::Match(tok1, "( %var% &&")) {
+            } else if (Token::Match(tok1, "(|%oror% ! %var% %oror%") ||
+                       Token::Match(tok1, "(|&& %var% &&")) {
                 // TODO: there are false negatives caused by this. The
                 // variable should be removed from skipvar after the
                 // condition
@@ -599,14 +623,16 @@ void CheckNullPointer::nullPointerStructByDeRefAndChec()
                     tok1 = tok1->next();
             }
 
-            // dereference in function call (but not sizeof|decltype)
-            else if ((Token::Match(tok1->tokAt(-2), "%var% ( %var% . %var%") && !Token::Match(tok1->tokAt(-2), "sizeof|decltype ( %var% . %var%")) ||
+            // dereference in function call (but not sizeof|decltype|typeof)
+            else if ((Token::Match(tok1->tokAt(-2), "%var% ( %var% . %var%") && !Token::Match(tok1->tokAt(-2), "sizeof|decltype|typeof ( %var% . %var%")) ||
                      Token::Match(tok1->previous(), ", %var% . %var%")) {
                 // Is the function return value taken by the pointer?
                 bool assignment = false;
                 const unsigned int varid1(tok1->varId());
                 if (varid1 == 0)
                     continue;
+                if (skipvar.find(varid1) != skipvar.end())
+                    continue;
                 const Token *tok2 = tok1->previous();
                 while (tok2 && !Token::Match(tok2, "[;{}]")) {
                     if (Token::Match(tok2, "%varid% =", varid1)) {
@@ -735,7 +761,7 @@ void CheckNullPointer::nullPointerByDeRefAndChec()
         // - logical operators
         const Token* tok = i->classDef;
         if ((i->type == Scope::eIf || i->type == Scope::eElseIf || i->type == Scope::eWhile) &&
-            tok && Token::Match(tok, "else| %var% ( !| %var% )|%oror%|&&") && !tok->next()->isExpandedMacro()) {
+            tok && Token::Match(tok, "else| %var% ( !| %var% )|%oror%|&&") && !tok->tokAt(tok->str()=="else"?1:0)->isExpandedMacro()) {
 
             if (tok->str() == "else")
                 tok = tok->next();
@@ -752,6 +778,19 @@ void CheckNullPointer::nullPointerByDeRefAndChec()
             // Variable id for pointer
             const unsigned int varid(vartok->varId());
 
+            // bailout for while scope if pointer is assigned inside the loop
+            if (i->type == Scope::eWhile) {
+                bool assign = false;
+                for (const Token *tok2 = i->classStart; tok2 && tok2 != i->classEnd; tok2 = tok2->next()) {
+                    if (Token::Match(tok2, "%varid% =", varid)) {
+                        assign = true;
+                        break;
+                    }
+                }
+                if (assign)
+                    continue;
+            }
+
             // Name of pointer
             const std::string& varname(vartok->str());
 
@@ -784,7 +823,7 @@ void CheckNullPointer::nullPointerByDeRefAndChec()
 
                     if (Token::Match(tok2->next(), "%var% ( %varid% ,", varid)) {
                         std::list<const Token *> varlist;
-                        parseFunctionCall(*(tok2->next()), varlist, 0);
+                        parseFunctionCall(*(tok2->next()), varlist, &_settings->library, 0);
                         if (!varlist.empty() && varlist.front() == tok2->tokAt(3)) {
                             nullPointerError(tok2->tokAt(3), varname, tok, inconclusive);
                             break;
@@ -813,7 +852,7 @@ void CheckNullPointer::nullPointerByDeRefAndChec()
                     // Don't write warning if the dereferencing is
                     // guarded by ?: or &&
                     const Token *tok2 = tok1->previous();
-                    if (tok2 && (tok2->isArithmeticalOp() || tok2->str() == "(")) {
+                    if (tok2 && (tok2->isArithmeticalOp() || Token::Match(tok2, "[(,]"))) {
                         while (tok2 && !Token::Match(tok2, "[;{}?:]")) {
                             if (tok2->str() == ")") {
                                 tok2 = tok2->link();
@@ -822,8 +861,8 @@ void CheckNullPointer::nullPointerByDeRefAndChec()
                                     break;
                                 }
                             }
-                            // guarded by &&
-                            if (tok2->varId() == varid && tok2->next()->str() == "&&")
+                            // guarded by && or ||
+                            if (Token::Match(tok2, "%varid% &&|%oror%",varid))
                                 break;
                             tok2 = tok2->previous();
                         }
@@ -1034,7 +1073,7 @@ void CheckNullPointer::nullPointerByCheckAndDeRef()
                 break;
 
             // parameters to sizeof are not dereferenced
-            if (Token::Match(tok2, "decltype|sizeof")) {
+            if (Token::Match(tok2, "decltype|sizeof|typeof")) {
                 if (tok2->strAt(1) != "(")
                     tok2 = tok2->next();
                 else
@@ -1045,7 +1084,7 @@ void CheckNullPointer::nullPointerByCheckAndDeRef()
             // function call, check if pointer is dereferenced
             if (Token::Match(tok2, "%var% (") && !Token::Match(tok2, "if|while")) {
                 std::list<const Token *> vars;
-                parseFunctionCall(*tok2, vars, 0);
+                parseFunctionCall(*tok2, vars, &_settings->library, 0);
                 for (std::list<const Token *>::const_iterator it = vars.begin(); it != vars.end(); ++it) {
                     if (Token::Match(*it, "%varid% [,)]", varid)) {
                         nullPointerError(*it, pointerName, vartok, inconclusive);
@@ -1103,9 +1142,8 @@ void CheckNullPointer::nullPointer()
         nullPointerStructByDeRefAndChec();
         nullPointerByDeRefAndChec();
         nullPointerByCheckAndDeRef();
+        nullPointerDefaultArgument();
     }
-
-    nullPointerDefaultArgument();
 }
 
 /** Dereferencing null constant (simplified token list) */
@@ -1121,11 +1159,11 @@ void CheckNullPointer::nullConstantDereference()
 
         const Token *tok = scope->classStart;
 
-        if (scope->function && (scope->function->type == Function::eConstructor || scope->function->type == Function::eCopyConstructor))
+        if (scope->function && scope->function->isConstructor())
             tok = scope->function->token; // Check initialization list
 
         for (; tok != scope->classEnd; tok = tok->next()) {
-            if (Token::Match(tok, "sizeof|decltype|typeid ("))
+            if (Token::Match(tok, "sizeof|decltype|typeid|typeof ("))
                 tok = tok->next()->link();
 
             else if (Token::simpleMatch(tok, "* 0")) {
@@ -1144,11 +1182,11 @@ void CheckNullPointer::nullConstantDereference()
                         nullPointerError(tok);
                 } else { // function call
                     std::list<const Token *> var;
-                    parseFunctionCall(*tok, var, 0);
+                    parseFunctionCall(*tok, var, &_settings->library, 0);
 
                     // is one of the var items a NULL pointer?
                     for (std::list<const Token *>::const_iterator it = var.begin(); it != var.end(); ++it) {
-                        if (Token::Match(*it, "0 [,)]")) {
+                        if (Token::Match(*it, "0|NULL [,)]")) {
                             nullPointerError(*it);
                         }
                     }
@@ -1172,7 +1210,7 @@ void CheckNullPointer::nullConstantDereference()
             }
 
             const Variable *ovar = 0;
-            if (Token::Match(tok, "0 ==|!= %var%"))
+            if (Token::Match(tok, "0 ==|!= %var% !!."))
                 ovar = tok->tokAt(2)->variable();
             else if (Token::Match(tok, "%var% ==|!= 0"))
                 ovar = tok->variable();
@@ -1184,6 +1222,26 @@ void CheckNullPointer::nullConstantDereference()
     }
 }
 
+/**
+* @brief If tok is a function call that passes in a pointer such that
+*         the pointer may be modified, this function will remove that
+*         pointer from pointerArgs.
+*/
+void CheckNullPointer::removeAssignedVarFromSet(const Token* tok, std::set<unsigned int>& pointerArgs)
+{
+    // If a pointer's address is passed into a function, stop considering it
+    if (Token::Match(tok->previous(), "[;{}] %var% (")) {
+        // Common functions that are known NOT to modify their pointer argument
+        const char safeFunctions[] = "printf|sprintf|fprintf|vprintf";
+
+        const Token* endParen = tok->next()->link();
+        for (const Token* tok2 = tok->next(); tok2 != endParen; tok2 = tok2->next()) {
+            if (tok2->isName() && tok2->varId() > 0 && !Token::Match(tok, safeFunctions)) {
+                pointerArgs.erase(tok2->varId());
+            }
+        }
+    }
+}
 
 /**
 * @brief Does one part of the check for nullPointer().
@@ -1213,8 +1271,8 @@ void CheckNullPointer::nullPointerDefaultArgument()
 
         // Report an error if any of the default-NULL arguments are dereferenced
         if (!pointerArgs.empty()) {
-            bool unknown = _settings->inconclusive;
             for (const Token *tok = scope->classStart; tok != scope->classEnd; tok = tok->next()) {
+
                 // If we encounter a possible NULL-pointer check, skip over its body
                 if (Token::simpleMatch(tok, "if ( "))  {
                     bool dependsOnPointer = false;
@@ -1231,31 +1289,53 @@ void CheckNullPointer::nullPointerDefaultArgument()
                     // pointer check for the pointers referenced in its condition
                     const Token *endOfIf = startOfIfBlock->link();
                     bool isExitOrReturn =
-                        Token::findmatch(startOfIfBlock, "exit|return", endOfIf) != NULL;
+                        Token::findmatch(startOfIfBlock, "exit|return|throw", endOfIf) != NULL;
 
-                    for (const Token *tok2 = tok->next(); tok2 != endOfCondition; tok2 = tok2->next()) {
-                        if (tok2->isName() && tok2->varId() > 0 &&
-                            pointerArgs.count(tok2->varId()) > 0) {
-
-                            // If the if() depends on a pointer and may return, stop
-                            // considering that pointer because it may be a NULL-pointer
-                            // check that returns if the pointer is NULL.
+                    if (Token::Match(tok, "if ( %var% == 0 )")) {
+                        const unsigned int var = tok->tokAt(2)->varId();
+                        if (var > 0 && pointerArgs.count(var) > 0) {
                             if (isExitOrReturn)
-                                pointerArgs.erase(tok2->varId());
+                                pointerArgs.erase(var);
                             else
                                 dependsOnPointer = true;
                         }
+                    } else {
+                        for (const Token *tok2 = tok->next(); tok2 != endOfCondition; tok2 = tok2->next()) {
+                            if (tok2->isName() && tok2->varId() > 0 &&
+                                pointerArgs.count(tok2->varId()) > 0) {
+
+                                // If the if() depends on a pointer and may return, stop
+                                // considering that pointer because it may be a NULL-pointer
+                                // check that returns if the pointer is NULL.
+                                if (isExitOrReturn)
+                                    pointerArgs.erase(tok2->varId());
+                                else
+                                    dependsOnPointer = true;
+                            }
+                        }
                     }
+
                     if (dependsOnPointer && endOfIf) {
                         for (; tok != endOfIf; tok = tok->next()) {
                             // If a pointer is assigned a new value, stop considering it.
                             if (Token::Match(tok, "%var% ="))
                                 pointerArgs.erase(tok->varId());
+                            else
+                                removeAssignedVarFromSet(tok, pointerArgs);
                         }
                         continue;
                     }
                 }
 
+                // If there is a noreturn function (e.g. exit()), stop considering the rest of
+                // this function.
+                bool unknown = false;
+                if (Token::Match(tok, "return|throw|exit") ||
+                    (_tokenizer->IsScopeNoReturn(tok, &unknown) && !unknown))
+                    break;
+
+                removeAssignedVarFromSet(tok, pointerArgs);
+
                 if (tok->varId() == 0 || pointerArgs.count(tok->varId()) == 0)
                     continue;
 
@@ -1266,7 +1346,7 @@ void CheckNullPointer::nullPointerDefaultArgument()
                 // If a pointer dereference is preceded by an && or ||,
                 // they serve as a sequence point so the dereference
                 // may not be executed.
-                if (isPointerDeRef(tok, unknown) &&
+                if (isPointerDeRef(tok, unknown) && !unknown &&
                     tok->strAt(-1) != "&&" && tok->strAt(-1) != "||" &&
                     tok->strAt(-2) != "&&" && tok->strAt(-2) != "||")
                     nullPointerDefaultArgError(tok, tok->str());
@@ -1286,16 +1366,18 @@ void CheckNullPointer::nullPointerDefaultArgument()
 class Nullpointer : public ExecutionPath {
 public:
     /** Startup constructor */
-    Nullpointer(Check *c, const SymbolDatabase* symbolDatabase_) : ExecutionPath(c, 0), symbolDatabase(symbolDatabase_), null(false) {
+    Nullpointer(Check *c, const SymbolDatabase* symbolDatabase_, const Library *lib) : ExecutionPath(c, 0), symbolDatabase(symbolDatabase_), library(lib), null(false) {
     }
 
 private:
     const SymbolDatabase* symbolDatabase;
+    const Library *library;
 
     /** Create checking of specific variable: */
-    Nullpointer(Check *c, const unsigned int id, const std::string &name, const SymbolDatabase* symbolDatabase_)
+    Nullpointer(Check *c, const unsigned int id, const std::string &name, const SymbolDatabase* symbolDatabase_, const Library *lib)
         : ExecutionPath(c, id),
           symbolDatabase(symbolDatabase_),
+          library(lib),
           varname(name),
           null(false) {
     }
@@ -1321,12 +1403,12 @@ private:
     bool null;
 
     /** variable is set to null */
-    static void setnull(std::list<ExecutionPath *> &checks, const unsigned int varid) {
+    static void setnull(std::list<ExecutionPath *> &checks, const unsigned int varid, bool null) {
         std::list<ExecutionPath *>::iterator it;
         for (it = checks.begin(); it != checks.end(); ++it) {
             Nullpointer *c = dynamic_cast<Nullpointer *>(*it);
             if (c && c->varId == varid)
-                c->null = true;
+                c->null = null;
         }
     }
 
@@ -1365,7 +1447,7 @@ private:
             // Pointer declaration declaration?
             const Variable *var = tok.variable();
             if (var && var->isPointer() && var->nameToken() == &tok)
-                checks.push_back(new Nullpointer(owner, var->varId(), var->name(), symbolDatabase));
+                checks.push_back(new Nullpointer(owner, var->declarationId(), var->name(), symbolDatabase, library));
         }
 
         if (Token::simpleMatch(&tok, "try {")) {
@@ -1385,7 +1467,7 @@ private:
 
             // parse usage..
             std::list<const Token *> var;
-            CheckNullPointer::parseFunctionCall(tok, var, 0);
+            CheckNullPointer::parseFunctionCall(tok, var, library, 0);
             for (std::list<const Token *>::const_iterator it = var.begin(); it != var.end(); ++it)
                 dereference(checks, *it);
         }
@@ -1403,8 +1485,8 @@ private:
                 dereference(checks, &tok);
             else if (unknown && owner->inconclusiveFlag())
                 dereference(checks, &tok);
-            if (Token::Match(tok.previous(), "[;{}=] %var% = 0 ;"))
-                setnull(checks, tok.varId());
+            if (Token::Match(tok.previous(), "[;{}=] %var% ="))
+                setnull(checks, tok.varId(), Token::simpleMatch(tok.tokAt(2), "0 ;"));
             else if (!deref &&
                      (!tok.previous()->isOp() || tok.previous()->str() == "&") &&
                      (!tok.next()->isConstOp() || tok.next()->str() == ">>"))
@@ -1453,7 +1535,7 @@ private:
 
         if (Token::Match(&tok, "!| %var% (")) {
             std::list<const Token *> var;
-            CheckNullPointer::parseFunctionCall(tok.str() == "!" ? *tok.next() : tok, var, 0);
+            CheckNullPointer::parseFunctionCall(tok.str() == "!" ? *tok.next() : tok, var, library, 0);
             for (std::list<const Token *>::const_iterator it = var.begin(); it != var.end(); ++it)
                 dereference(checks, *it);
         }
@@ -1479,7 +1561,7 @@ private:
 void CheckNullPointer::executionPaths()
 {
     // Check for null pointer errors..
-    Nullpointer c(this, _tokenizer->getSymbolDatabase());
+    Nullpointer c(this, _tokenizer->getSymbolDatabase(), &_settings->library);
     checkExecutionPaths(_tokenizer->getSymbolDatabase(), &c);
 }
 
@@ -1499,7 +1581,7 @@ void CheckNullPointer::nullPointerError(const Token *tok, const std::string &var
     callstack.push_back(tok);
     callstack.push_back(nullCheck);
     const std::string errmsg("Possible null pointer dereference: " + varname + " - otherwise it is redundant to check it against null.");
-    reportError(callstack, Severity::error, "nullPointer", errmsg, inconclusive);
+    reportError(callstack, Severity::warning, "nullPointer", errmsg, inconclusive);
 }
 
 void CheckNullPointer::nullPointerDefaultArgError(const Token *tok, const std::string &varname)
