@@ -39,13 +39,13 @@ class Variable;
 class CPPCHECKLIB CheckUninitVar : public Check {
 public:
     /** @brief This constructor is used when registering the CheckUninitVar */
-    CheckUninitVar() : Check(myName())
-    { }
+    CheckUninitVar() : Check(myName()), testrunner(false) {
+    }
 
     /** @brief This constructor is used when running checks. */
     CheckUninitVar(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger)
-        : Check(myName(), tokenizer, settings, errorLogger)
-    { }
+        : Check(myName(), tokenizer, settings, errorLogger), testrunner(false) {
+    }
 
     /** @brief Run checks against the simplified token list */
     void runSimplifiedChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) {
@@ -57,12 +57,14 @@ public:
     /** Check for uninitialized variables */
     void check();
     void checkScope(const Scope* scope);
-    bool checkScopeForVariable(const Scope* scope, const Token *tok, const Variable& var, bool * const possibleInit, bool * const noreturn, const std::string &membervar);
-    bool checkIfForWhileHead(const Token *startparentheses, const Variable& var, bool suppressErrors, bool isuninit, const std::string &membervar);
-    bool checkLoopBody(const Token *tok, const Variable& var, const std::string &membervar, const bool suppressErrors);
-    static bool isVariableUsage(const Token *vartok, bool ispointer, bool cpp);
-    bool isMemberVariableAssignment(const Token *tok, const std::string &membervar) const;
-    bool isMemberVariableUsage(const Token *tok, bool isPointer, const std::string &membervar) const;
+    void checkStruct(const Scope* scope, const Token *tok, const Variable &structvar);
+    bool checkScopeForVariable(const Scope* scope, const Token *tok, const Variable& var, bool * const possibleInit, bool * const noreturn, bool * const alloc, const std::string &membervar);
+    bool checkIfForWhileHead(const Token *startparentheses, const Variable& var, bool suppressErrors, bool isuninit, bool alloc, const std::string &membervar);
+    bool checkLoopBody(const Token *tok, const Variable& var, const bool alloc, const std::string &membervar, const bool suppressErrors);
+    void checkRhs(const Token *tok, const Variable &var, bool alloc, const std::string &membervar);
+    static bool isVariableUsage(const Token *vartok, bool ispointer, bool alloc, bool cpp);
+    static bool isMemberVariableAssignment(const Token *tok, const std::string &membervar);
+    bool isMemberVariableUsage(const Token *tok, bool isPointer, bool alloc, const std::string &membervar) const;
 
     /**
      * @brief Uninitialized variables: analyse functions to see how they work with uninitialized variables
@@ -82,6 +84,9 @@ public:
     void uninitvarError(const Token *tok, const std::string &varname);
     void uninitStructMemberError(const Token *tok, const std::string &membername);
 
+    /** testrunner: (don't abort() when assertion fails, just write error message) */
+    bool testrunner;
+
 private:
     void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) const {
         CheckUninitVar c(0, settings, errorLogger);
@@ -104,4 +109,4 @@ private:
 };
 /// @}
 //---------------------------------------------------------------------------
-#endif
+#endif // checkuninitvarH
