@@ -22,6 +22,7 @@
 #include "symboldatabase.h"
 #include <algorithm>
 #include <cctype>
+#include <utility>
 //---------------------------------------------------------------------------
 
 // Register this check class (by creating a static instance of it)
@@ -56,7 +57,7 @@ public:
 
         /** variable is used.. set both read+write */
         void use(std::list<std::set<unsigned int> > & varReadInScope) {
-            varReadInScope.back().insert(_var->varId());
+            varReadInScope.back().insert(_var->declarationId());
             _read = true;
             _write = true;
         }
@@ -160,7 +161,7 @@ void Variables::alias(unsigned int varid1, unsigned int varid2, bool replace)
             VariableUsage *temp = find(*i);
 
             if (temp)
-                temp->_aliases.erase(var1->_var->varId());
+                temp->_aliases.erase(var1->_var->declarationId());
         }
 
         // remove all aliases from var1
@@ -195,7 +196,7 @@ void Variables::clearAliases(unsigned int varid)
             VariableUsage *temp = find(*i);
 
             if (temp)
-                temp->_aliases.erase(usage->_var->varId());
+                temp->_aliases.erase(usage->_var->declarationId());
         }
 
         // remove all aliases from usage
@@ -225,9 +226,9 @@ void Variables::addVar(const Variable *var,
                        VariableType type,
                        bool write_)
 {
-    if (var->varId() > 0) {
-        _varAddedInScope.back().insert(var->varId());
-        _varUsage.insert(std::make_pair(var->varId(), VariableUsage(var, type, false, write_, false)));
+    if (var->declarationId() > 0) {
+        _varAddedInScope.back().insert(var->declarationId());
+        _varUsage.insert(std::make_pair(var->declarationId(), VariableUsage(var, type, false, write_, false)));
     }
 }
 
@@ -461,6 +462,8 @@ static const Token* doAssignment(Variables &variables, const Token *tok, bool de
                         addressOf = true;
                         tok = tok->next();
                     }
+                } else if (Token::Match(tok, "%cop% %var%")) {
+                    variables.read(tok->next()->varId(), tok);
                 }
             }
 
@@ -618,29 +621,15 @@ static const Token* doAssignment(Variables &variables, const Token *tok, bool de
     return tok;
 }
 
-static bool isRecordTypeWithoutSideEffects(const Type* type)
-{
-    // a type that has no side effects (no constructors and no members with constructors)
-    /** @todo false negative: check constructors for side effects */
-    if (type && type->classScope && type->classScope->numConstructors == 0 &&
-        (type->classScope->varlist.empty() || type->needInitialization == Type::True)) {
-        bool yes = true;
-        for (std::vector<Type::BaseInfo>::const_iterator i = type->derivedFrom.begin(); yes && i != type->derivedFrom.end(); ++i)
-            yes = isRecordTypeWithoutSideEffects(i->type);
-        return yes;
-    }
-    return false;
-}
-
 static bool isPartOfClassStructUnion(const Token* tok)
 {
     for (; tok; tok = tok->previous()) {
         if (tok->str() == "}" || tok->str() == ")")
             tok = tok->link();
         else if (tok->str() == "(")
-            return(false);
+            return (false);
         else if (tok->str() == "{") {
-            return(tok->strAt(-1) == "struct" || tok->strAt(-2) == "struct" || tok->strAt(-1) == "class" || tok->strAt(-2) == "class" || tok->strAt(-1) == "union" || tok->strAt(-2) == "union");
+            return (tok->strAt(-1) == "struct" || tok->strAt(-2) == "struct" || tok->strAt(-1) == "class" || tok->strAt(-2) == "class" || tok->strAt(-1) == "union" || tok->strAt(-2) == "union");
         }
     }
     return false;
@@ -718,7 +707,7 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
                 }
             }
             if (i->isArray() && i->isClass()) // Array of class/struct members. Initialized by ctor.
-                variables.write(i->varId(), i->nameToken());
+                variables.write(i->declarationId(), i->nameToken());
             if (i->isArray() && Token::Match(i->nameToken(), "%var% [ %var% ]")) // Array index variable read.
                 variables.read(i->nameToken()->tokAt(2)->varId(), i->nameToken());
 
@@ -791,6 +780,24 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
             }
         }
 
+        // C++11 std::for_each
+        // No warning should be written if a variable is first read and
+        // then written in the body.
+        if (Token::simpleMatch(tok, "for_each (") && Token::simpleMatch(tok->linkAt(1), ") ;")) {
+            const Token *end = tok->linkAt(1);
+            if (end->previous()->str() == "}") {
+                std::set<unsigned int> readvar;
+                for (const Token *body = end->linkAt(-1); body != end; body = body->next()) {
+                    if (body->varId() == 0U)
+                        continue;
+                    if (!Token::Match(body->next(),"="))
+                        readvar.insert(body->varId());
+                    else if (readvar.find(body->varId()) != readvar.end())
+                        variables.erase(body->varId());
+                }
+            }
+        }
+
         if (Token::Match(tok->previous(), "[;{}]")) {
             for (const Token* tok2 = tok->next(); tok2; tok2 = tok2->next()) {
                 if (tok2->varId()) {
@@ -989,8 +996,11 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
                 variables.read(tok->next()->varId(), tok);
             } else // addressof
                 variables.use(tok->next()->varId(), tok); // use = read + write
-        } else if (Token::Match(tok, ">> %var%")) {
-            variables.use(tok->next()->varId(), tok); // use = read + write
+        } else if (Token::Match(tok, ">>|>>= %var%")) {
+            if (_tokenizer->isC() || tok->previous()->isStandardType())
+                variables.read(tok->next()->varId(), tok);
+            else
+                variables.use(tok->next()->varId(), tok); // use = read + write
         } else if (Token::Match(tok, "%var% >>|&") && Token::Match(tok->previous(), "[{};:]")) {
             variables.read(tok->varId(), tok);
         }
@@ -1089,7 +1099,7 @@ void CheckUnusedVar::checkFunctionVariableUsage()
             const Variable* var = symbolDatabase->getVariableFromVarId(it->first);
 
             // variable has been marked as unused so ignore it
-            if (usage._var->nameToken()->isUnused())
+            if (usage._var->nameToken()->isAttributeUnused())
                 continue;
 
             // skip things that are only partially implemented to prevent false positives
@@ -1254,3 +1264,30 @@ void CheckUnusedVar::unusedStructMemberError(const Token *tok, const std::string
 {
     reportError(tok, Severity::style, "unusedStructMember", "struct or union member '" + structname + "::" + varname + "' is never used.");
 }
+
+bool CheckUnusedVar::isRecordTypeWithoutSideEffects(const Type* type)
+{
+    // a type that has no side effects (no constructors and no members with constructors)
+    /** @todo false negative: check constructors for side effects */
+
+    std::pair<std::map<Type const *,bool>::iterator,bool> found=isRecordTypeWithoutSideEffectsMap.insert(
+                std::pair<const Type *,bool>(type,false)); //Initialize with side effects for possilbe recursions
+    bool & withoutSideEffects=found.first->second;
+    if (!found.second)
+        return withoutSideEffects;
+
+    if (type && type->classScope && type->classScope->numConstructors == 0 &&
+        (type->classScope->varlist.empty() || type->needInitialization == Type::True)) {
+        for (std::vector<Type::BaseInfo>::const_iterator i = type->derivedFrom.begin(); i != type->derivedFrom.end(); ++i) {
+            if (!isRecordTypeWithoutSideEffects(i->type)) {
+                withoutSideEffects=false;
+                return withoutSideEffects;
+            }
+        }
+        withoutSideEffects=true;
+        return withoutSideEffects;
+    }
+
+    withoutSideEffects=false;   // unknown types are assumed to have side effects
+    return withoutSideEffects;
+}
