@@ -35,11 +35,12 @@ Settings::Settings()
       _xml(false), _xml_version(1),
       _jobs(1),
       _exitCode(0),
-      _showtime(0),
+      _showtime(SHOWTIME_NONE),
       _maxConfigs(12),
       enforcedLang(None),
       reportProgress(false),
-      checkConfiguration(false)
+      checkConfiguration(false),
+      checkLibrary(false)
 {
     // This assumes the code you are checking is for the same architecture this is compiled on.
 #if defined(_WIN64)
