@@ -16,8 +16,10 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef TokenH
-#define TokenH
+//---------------------------------------------------------------------------
+#ifndef tokenH
+#define tokenH
+//---------------------------------------------------------------------------
 
 #include <string>
 #include <vector>
@@ -145,15 +147,15 @@ public:
      * - "!!else" No tokens or any token that is not "else".
      * - "someRandomText" If token contains "someRandomText".
      *
-     * multi-compare patterns such as "int|void|char" can contain %or%, %oror% and %op%
-     * but it is not recommended to put such an %cmd% as the first pattern.
+     * multi-compare patterns such as "int|void|char" can contain %%or%, %%oror% and %%op%
+     * but it is not recommended to put such an %%cmd% as the first pattern.
      *
-     * It's possible to use multi-compare patterns with all the other %cmds%,
-     * except for %varid%, and normal names, but the %cmds% should be put as
+     * It's possible to use multi-compare patterns with all the other %%cmds%,
+     * except for %%varid%, and normal names, but the %%cmds% should be put as
      * the first patterns in the list, then the normal names.
      * For example: "%var%|%num%|)" means yes to a variable, a number or ')'.
      *
-     * @todo Make it possible to use the %cmds% and the normal names in the
+     * @todo Make it possible to use the %%cmds% and the normal names in the
      * multicompare list without an order.
      *
      * The patterns can be also combined to compare to multiple tokens at once
@@ -165,7 +167,7 @@ public:
      * @param tok List of tokens to be compared to the pattern
      * @param pattern The pattern against which the tokens are compared,
      * e.g. "const" or ") const|volatile| {".
-     * @param varid if %varid% is given in the pattern the Token::varId
+     * @param varid if %%varid% is given in the pattern the Token::varId
      * will be matched against this argument
      * @return true if given token matches with given pattern
      *         false if given token does not match with given pattern
@@ -175,7 +177,7 @@ public:
     /**
      * Return length of C-string.
      *
-     * Should be called for %str% tokens only.
+     * Should be called for %%str%% tokens only.
      *
      * @param tok token with C-string
      **/
@@ -184,7 +186,7 @@ public:
     /**
      * Return char of C-string at index (possible escaped "\\n")
      *
-     * Should be called for %str% tokens only.
+     * Should be called for %%str%% tokens only.
      *
      * @param tok token with C-string
      * @param index position of character
@@ -202,6 +204,10 @@ public:
                _type == eBoolean; // TODO: "true"/"false" aren't really a name...
     }
     bool isUpperCaseName() const;
+    bool isLiteral() const {
+        return _type == eNumber || _type == eString || _type == eChar ||
+               _type == eBoolean || _type == eLiteral;
+    }
     bool isNumber() const {
         return _type == eNumber;
     }
@@ -257,21 +263,27 @@ public:
     void isLong(bool size) {
         _isLong = size;
     }
-    bool isUnused() const {
-        return _isUnused;
-    }
-    void isUnused(bool used) {
-        _isUnused = used;
-    }
     bool isStandardType() const {
         return _isStandardType;
     }
     bool isExpandedMacro() const {
         return _isExpandedMacro;
     }
-    void setExpandedMacro(bool m) {
+    void isExpandedMacro(bool m) {
         _isExpandedMacro = m;
     }
+    bool isAttributeConstructor() const {
+        return _isAttributeConstructor;
+    }
+    void isAttributeConstructor(bool ac) {
+        _isAttributeConstructor = ac;
+    }
+    bool isAttributeUnused() const {
+        return _isAttributeUnused;
+    }
+    void isAttributeUnused(bool unused) {
+        _isAttributeUnused = unused;
+    }
 
     static const Token *findsimplematch(const Token *tok, const char pattern[]);
     static const Token *findsimplematch(const Token *tok, const char pattern[], const Token *end);
@@ -343,6 +355,8 @@ public:
      */
     void insertToken(const std::string &tokenStr, bool prepend=false);
 
+    void insertToken(const std::string &tokenStr, const std::string &originalNameStr, bool prepend=false);
+
     Token *previous() const {
         return _previous;
     }
@@ -542,15 +556,23 @@ public:
     /**
      * Returns the closing bracket of opening '<'. Should only be used if link()
      * is unavailable.
-     * @param closing The closing token is stored in that parameter
-     * @return success
+     * @return closing '>', ')', ']' or '}'. if no closing bracket is found, NULL is returned
      */
-    bool findClosingBracket(const Token*& closing) const;
-    bool findClosingBracket(Token*& closing) const {
-        const Token* tok;
-        bool retVal = findClosingBracket(tok);
-        closing = const_cast<Token*>(tok);
-        return retVal;
+    const Token* findClosingBracket() const;
+    Token* findClosingBracket();
+
+    /**
+     * Returns the original name.
+     */
+    const std::string & originalName() const {
+        return _originalName;
+    }
+
+    /**
+     * Sets the original name.
+     */
+    void originalName(const std::string & name) {
+        _originalName = name;
     }
 
 private:
@@ -610,9 +632,10 @@ private:
     bool _isSigned;
     bool _isPointerCompare;
     bool _isLong;
-    bool _isUnused;
     bool _isStandardType;
     bool _isExpandedMacro;
+    bool _isAttributeConstructor;  // __attribute__((constructor))
+    bool _isAttributeUnused;       // __attribute__((unused))
 
     /** Updates internal property cache like _isName or _isBoolean.
         Called after any _str() modification. */
@@ -625,11 +648,13 @@ private:
     Token *_astOperand1;
     Token *_astOperand2;
     Token *_astParent;
+
+    // original name like size_t
+    std::string _originalName;
+
 public:
     void astOperand1(Token *tok);
     void astOperand2(Token *tok);
-    void astFunctionCall();
-    void astHandleParentheses();
 
     const Token * astOperand1() const {
         return _astOperand1;
@@ -637,6 +662,9 @@ public:
     const Token * astOperand2() const {
         return _astOperand2;
     }
+    const Token * astParent() const {
+        return _astParent;
+    }
     const Token *astTop() const {
         const Token *ret = this;
         while (ret->_astParent)
@@ -644,16 +672,31 @@ public:
         return ret;
     }
 
-    std::string astString() const {
+    /**
+     * Is current token a calculation? Only true for operands.
+     * For '*' and '&' tokens it is looked up if this is a
+     * dereference or address-of. A dereference or address-of is not
+     * counted as a calculation.
+     * @return returns true if current token is a calculation
+     */
+    bool isCalculation() const;
+
+    void clearAst() {
+        _astOperand1 = _astOperand2 = _astParent = NULL;
+    }
+
+    std::string astString(const char *sep = "") const {
         std::string ret;
         if (_astOperand1)
-            ret = _astOperand1->astString();
+            ret = _astOperand1->astString(sep);
         if (_astOperand2)
-            ret += _astOperand2->astString();
-        return ret+_str;
+            ret += _astOperand2->astString(sep);
+        return ret + sep + _str;
     }
+
+    void printAst() const;
 };
 
 /// @}
-
-#endif // TokenH
+//---------------------------------------------------------------------------
+#endif // tokenH
