@@ -16,7 +16,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 //---------------------------------------------------------------------------
 #ifndef tokenizeH
 #define tokenizeH
@@ -29,6 +28,7 @@
 #include <string>
 #include <map>
 #include <list>
+#include <ctime>
 
 class Settings;
 class SymbolDatabase;
@@ -63,7 +63,7 @@ public:
      * \param unknown set to true if it's unknown if the scope is noreturn
      * \return true if scope ends with a function call that might be 'noreturn'
      */
-    static bool IsScopeNoReturn(const Token *endScopeToken, bool *unknown = 0);
+    bool IsScopeNoReturn(const Token *endScopeToken, bool *unknown = 0) const;
 
     /**
      * Tokenize code
@@ -101,12 +101,20 @@ public:
     void setVarId();
 
     /**
-     * Simplify tokenlist
-     *
-     * @return false if there is an error that requires aborting
-     * the checking of this file.
-     */
-    bool simplifyTokenList();
+    * Basic simplification of tokenlist
+    *
+    * @return false if there is an error that requires aborting
+    * the checking of this file.
+    */
+    bool simplifyTokenList1();
+
+    /**
+    * Most aggressive simplification of tokenlist
+    *
+    * @return false if there is an error that requires aborting
+    * the checking of this file.
+    */
+    bool simplifyTokenList2();
 
     /**
      * Deletes dead code between 'begin' and 'end'.
@@ -165,6 +173,13 @@ public:
      */
     bool simplifyCalculations();
 
+    /**
+     * Simplify dereferencing a pointer offset by a number:
+     *     "*(ptr + num)" => "ptr[num]"
+     *     "*(ptr - num)" => "ptr[-num]"
+     */
+    void simplifyOffsetPointerDereference();
+
     /** Insert array size where it isn't given */
     void arraySize();
 
@@ -189,6 +204,9 @@ public:
       */
     void simplifyRealloc();
 
+    /** Add parentheses for sizeof: sizeof x => sizeof(x) */
+    void sizeofAddParentheses();
+
     /**
      * Replace sizeof() to appropriate size.
      * @return true if modifications to token-list are done.
@@ -201,6 +219,7 @@ public:
      * \param only_k_r_fpar Only simplify K&R function parameters
      */
     void simplifyVarDecl(bool only_k_r_fpar);
+    void simplifyVarDecl(Token * tokBegin, Token * tokEnd, bool only_k_r_fpar);
 
     /**
      * Simplify variable initialization
@@ -237,10 +256,12 @@ public:
     void simplifyCompoundAssignment();
 
     /**
-     * simplify if-assignments
+     * Simplify assignments in "if" and "while" conditions
      * Example: "if(a=b);" => "a=b;if(a);"
+     * Example: "while(a=b) { f(a); }" => "a = b; while(a){ f(a); a = b; }"
+     * Example: "do { f(a); } while(a=b);" => "do { f(a); a = b; } while(a);"
      */
-    void simplifyIfAssign();
+    void simplifyIfAndWhileAssign();
 
     /**
      * Simplify multiple assignments.
@@ -312,6 +333,11 @@ public:
      */
     void simplifyTypedef();
 
+    /**
+     * Simplify float casts (float)1 => 1.0
+     */
+    void simplifyFloatCasts();
+
     /**
      * Simplify casts
      */
@@ -471,8 +497,9 @@ public:
 
     /**
      * Simplify e.g. 'atol("0")' into '0'
+     * @return returns true if simplifcations performed and false otherwise.
      */
-    void simplifyMathFunctions();
+    bool simplifyMathFunctions();
 
     /**
      * Simplify e.g. 'sin(0)' into '0'
@@ -561,7 +588,7 @@ public:
     void syntaxError(const Token *tok, char c) const;
 
     /** Report that there is an unhandled "class x y {" code */
-    void unhandled_macro_class_x_y(const Token *tok);
+    void unhandled_macro_class_x_y(const Token *tok) const;
 
     /**
      * assert that tokens are ok - used during debugging for example
@@ -739,6 +766,27 @@ public:
      */
     static Token *copyTokens(Token *dest, const Token *first, const Token *last, bool one_line = true);
 
+    /**
+    * Helper function to check wether number is zero (0 or 0.0 or 0E+0) or not?
+    * @param s --> a string to check
+    * @return true in case is is zero and false otherwise.
+    */
+    static bool isZeroNumber(const std::string &s);
+
+    /**
+    * Helper function to check wether number is one (1 or 0.1E+1 or 1E+0) or not?
+    * @param s --> a string to check
+    * @return true in case is is one and false otherwise.
+    */
+    static bool isOneNumber(const std::string &s);
+
+    /**
+    * Helper function to check wether number is one (2 or 0.2E+1 or 2E+0) or not?
+    * @param s --> a string to check
+    * @return true in case is is two and false otherwise.
+    */
+    static bool isTwoNumber(const std::string &s);
+
 private:
     /** Disable copy constructor, no implementation */
     Tokenizer(const Tokenizer &);
@@ -775,9 +823,13 @@ private:
      * TimerResults
      */
     TimerResults *m_timerResults;
+#ifdef MAXTIME
+    /** Tokenizer maxtime */
+    std::time_t maxtime;
+#endif
 };
 
 /// @}
 
 //---------------------------------------------------------------------------
-#endif
+#endif // tokenizeH
