@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,8 +27,8 @@ extern std::ostringstream errout;
 
 class TestConstructors : public TestFixture {
 public:
-    TestConstructors() : TestFixture("TestConstructors")
-    { }
+    TestConstructors() : TestFixture("TestConstructors") {
+    }
 
 private:
 
@@ -40,12 +40,13 @@ private:
         Settings settings;
         settings.inconclusive = showAll;
         settings.addEnabled("style");
+        settings.addEnabled("warning");
 
         // Tokenize..
         Tokenizer tokenizer(&settings, this);
         std::istringstream istr(code);
         tokenizer.tokenize(istr, "test.cpp");
-        tokenizer.simplifyTokenList();
+        tokenizer.simplifyTokenList2();
 
         // Check class constructors..
         CheckClass checkClass(&tokenizer, &settings, this);
@@ -59,6 +60,12 @@ private:
         TEST_CASE(simple4);
         TEST_CASE(simple5); // ticket #2560
         TEST_CASE(simple6); // ticket #4085 - uninstantiated template class
+        TEST_CASE(simple7); // ticket #4531
+        TEST_CASE(simple8);
+        TEST_CASE(simple9); // ticket #4574
+        TEST_CASE(simple10); // ticket #4388
+        TEST_CASE(simple11); // ticket #4536
+        TEST_CASE(simple12); // ticket #4620
 
         TEST_CASE(initvar_with_this);       // BUG 2190300
         TEST_CASE(initvar_if);              // BUG 2190290
@@ -73,6 +80,7 @@ private:
         TEST_CASE(initvar_constvar);
         TEST_CASE(initvar_staticvar);
         TEST_CASE(initvar_union);
+        TEST_CASE(initvar_delegate);       // ticket #4302
 
         TEST_CASE(initvar_private_constructor);     // BUG 2354171 - private constructor
         TEST_CASE(initvar_copy_constructor); // ticket #1611
@@ -80,8 +88,10 @@ private:
         TEST_CASE(initvar_nocopy1);            // ticket #2474
         TEST_CASE(initvar_nocopy2);            // ticket #2484
         TEST_CASE(initvar_nocopy3);            // ticket #3611
+        TEST_CASE(initvar_with_member_function_this); // ticket #4824
 
         TEST_CASE(initvar_destructor);      // No variables need to be initialized in a destructor
+        TEST_CASE(initvar_func_ret_func_ptr); // ticket #4449
 
         TEST_CASE(operatorEqSTL);
 
@@ -107,6 +117,11 @@ private:
         TEST_CASE(uninitVar20); // ticket #2867
         TEST_CASE(uninitVar21); // ticket #2947
         TEST_CASE(uninitVar22); // ticket #3043
+        TEST_CASE(uninitVar23); // ticket #3702
+        TEST_CASE(uninitVar24); // ticket #3190
+        TEST_CASE(uninitVar25); // ticket #4789
+        TEST_CASE(uninitVar26);
+        TEST_CASE(uninitVar27); // ticket #5170 - rtl::math::setNan(&d)
         TEST_CASE(uninitVarEnum);
         TEST_CASE(uninitVarStream);
         TEST_CASE(uninitVarTypedef);
@@ -121,7 +136,8 @@ private:
         TEST_CASE(uninitVarArray8);
         TEST_CASE(uninitVarArray2D);
         TEST_CASE(uninitVarArray3D);
-        TEST_CASE(uninitVarCpp11Init);
+        TEST_CASE(uninitVarCpp11Init1);
+        TEST_CASE(uninitVarCpp11Init2);
         TEST_CASE(uninitVarStruct1);       // ticket #2172
         TEST_CASE(uninitVarStruct2);       // ticket #838
         TEST_CASE(uninitVarUnion1);        // ticket #3196
@@ -153,21 +169,21 @@ private:
               "{\n"
               "public:\n"
               "    int i;\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("", errout.str());
 
         check("class Fred\n"
               "{\n"
               "private:\n"
               "    int i;\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("[test.cpp:1]: (style) The class 'Fred' does not have a constructor.\n", errout.str());
 
         check("struct Fred\n"
               "{\n"
               "private:\n"
               "    int i;\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("[test.cpp:1]: (style) The struct 'Fred' does not have a constructor.\n", errout.str());
     }
 
@@ -177,32 +193,44 @@ private:
               "{\n"
               "public:\n"
               "    Fred() : i(0) { }\n"
+              "    Fred(Fred const & other) : i(other.i) {}\n"
+              "    Fred(Fred && other) : i(other.i) {}\n"
               "    int i;\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("", errout.str());
 
         check("class Fred\n"
               "{\n"
               "public:\n"
               "    Fred() { i = 0; }\n"
+              "    Fred(Fred const & other) {i=other.i}\n"
+              "    Fred(Fred && other) {i=other.i}\n"
               "    int i;\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("", errout.str());
 
         check("class Fred\n"
               "{\n"
               "public:\n"
               "    Fred() { }\n"
+              "    Fred(Fred const & other) {}\n"
+              "    Fred(Fred && other) {}\n"
               "    int i;\n"
-              "};\n");
-        ASSERT_EQUALS("[test.cpp:4]: (warning) Member variable 'Fred::i' is not initialized in the constructor.\n", errout.str());
+              "};");
+        ASSERT_EQUALS("[test.cpp:4]: (warning) Member variable 'Fred::i' is not initialized in the constructor.\n"
+                      "[test.cpp:5]: (warning) Member variable 'Fred::i' is not initialized in the constructor.\n"
+                      "[test.cpp:6]: (warning) Member variable 'Fred::i' is not initialized in the constructor.\n", errout.str());
 
         check("struct Fred\n"
               "{\n"
               "    Fred() { }\n"
+              "    Fred(Fred const & other) {}\n"
+              "    Fred(Fred && other) {}\n"
               "    int i;\n"
-              "};\n");
-        ASSERT_EQUALS("[test.cpp:3]: (warning) Member variable 'Fred::i' is not initialized in the constructor.\n", errout.str());
+              "};");
+        ASSERT_EQUALS("[test.cpp:3]: (warning) Member variable 'Fred::i' is not initialized in the constructor.\n"
+                      "[test.cpp:4]: (warning) Member variable 'Fred::i' is not initialized in the constructor.\n"
+                      "[test.cpp:5]: (warning) Member variable 'Fred::i' is not initialized in the constructor.\n", errout.str());
     }
 
 
@@ -213,7 +241,7 @@ private:
               "    int i;\n"
               "};\n"
               "Fred::Fred() :i(0)\n"
-              "{ }\n");
+              "{ }");
         ASSERT_EQUALS("", errout.str());
 
         check("struct Fred\n"
@@ -222,7 +250,7 @@ private:
               "    int i;\n"
               "};\n"
               "Fred::Fred()\n"
-              "{ i = 0; }\n");
+              "{ i = 0; }");
         ASSERT_EQUALS("", errout.str());
 
         check("struct Fred\n"
@@ -231,7 +259,7 @@ private:
               "    int i;\n"
               "};\n"
               "Fred::Fred()\n"
-              "{ }\n");
+              "{ }");
         ASSERT_EQUALS("[test.cpp:6]: (warning) Member variable 'Fred::i' is not initialized in the constructor.\n", errout.str());
     }
 
@@ -241,6 +269,7 @@ private:
               "{\n"
               "    Fred();\n"
               "    explicit Fred(int _i);\n"
+              "    Fred(Fred const & other);\n"
               "    int i;\n"
               "};\n"
               "Fred::Fred()\n"
@@ -248,8 +277,8 @@ private:
               "Fred::Fred(int _i)\n"
               "{\n"
               "    i = _i;\n"
-              "}\n");
-        ASSERT_EQUALS("[test.cpp:7]: (warning) Member variable 'Fred::i' is not initialized in the constructor.\n", errout.str());
+              "}\n", true);
+        ASSERT_EQUALS("[test.cpp:8]: (warning, inconclusive) Member variable 'Fred::i' is not initialized in the constructor.\n", errout.str());
     }
 
     void simple5() { // ticket #2560
@@ -272,10 +301,96 @@ private:
     void simple6() { // ticket #4085 - uninstantiated template class
         check("template <class T> struct A {\n"
               "    A<T>() { x = 0; }\n"
+              "    A<T>(const T & t) { x = t.x; }\n"
               "private:\n"
               "    int x;\n"
               "};");
         ASSERT_EQUALS("", errout.str());
+
+        check("template <class T> struct A {\n"
+              "    A<T>() : x(0) { }\n"
+              "    A<T>(const T & t) : x(t.x) { }\n"
+              "private:\n"
+              "    int x;\n"
+              "};");
+        ASSERT_EQUALS("", errout.str());
+
+        check("template <class T> struct A {\n"
+              "    A<T>() : x(0) { }\n"
+              "    A<T>(const T & t) : x(t.x) { }\n"
+              "private:\n"
+              "    int x;\n"
+              "    int y;\n"
+              "};");
+        ASSERT_EQUALS("[test.cpp:2]: (warning) Member variable 'A::y' is not initialized in the constructor.\n"
+                      "[test.cpp:3]: (warning) Member variable 'A::y' is not initialized in the constructor.\n", errout.str());
+    }
+
+    void simple7() { // ticket #4531
+        check("class Fred;\n"
+              "struct Fred {\n"
+              "    int x;\n"
+              "};");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void simple8() {
+        check("struct Fred { int x; };\n"
+              "class Barney { Fred fred; };\n"
+              "class Wilma { struct Betty { int x; } betty; };");
+        ASSERT_EQUALS("[test.cpp:2]: (style) The class 'Barney' does not have a constructor.\n"
+                      "[test.cpp:3]: (style) The class 'Wilma' does not have a constructor.\n", errout.str());
+    }
+
+    void simple9() { // ticket #4574
+        check("class Unknown::Fred {\n"
+              "public:\n"
+              "    Fred() : x(0) { }\n"
+              "private:\n"
+              "    int x;\n"
+              "};");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void simple10() { // ticket #4388
+        check("class Fred {\n"
+              "public:\n"
+              "    Fred() = default;\n"
+              "private:\n"
+              "    int x;\n"
+              "};");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void simple11() { // ticket #4536
+        check("class Fred {\n"
+              "public:\n"
+              "    Fred() {}\n"
+              "private:\n"
+              "    int x = 0;\n"
+              "};");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void simple12() { // ticket #4620
+        check("class Fred {\n"
+              "    int x;\n"
+              "public:\n"
+              "    Fred() { Init(); }\n"
+              "    void Init(int i = 0);\n"
+              "};\n"
+              "void Fred::Init(int i) { x = i; }");
+        ASSERT_EQUALS("", errout.str());
+
+        check("class Fred {\n"
+              "    int x;\n"
+              "    int y;\n"
+              "public:\n"
+              "    Fred() { Init(0); }\n"
+              "    void Init(int i, int j = 0);\n"
+              "};\n"
+              "void Fred::Init(int i, int j) { x = i; y = j; }");
+        ASSERT_EQUALS("", errout.str());
     }
 
     void initvar_with_this() {
@@ -284,7 +399,7 @@ private:
               "    Fred()\n"
               "    { this->i = 0; }\n"
               "    int i;\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -299,7 +414,7 @@ private:
               "            i = 1;\n"
               "    }\n"
               "    int i;\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -355,7 +470,7 @@ private:
               "    Fred() { i = 0; }\n"
               "    void operator=(const Fred &fred) { }\n"
               "    int i;\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("[test.cpp:4]: (warning) Member variable 'Fred::i' is not assigned a value in 'Fred::operator='.\n", errout.str());
     }
 
@@ -367,7 +482,7 @@ private:
               "private:\n"
               "    void Init() { i = 0; }\n"
               "    int i;\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -384,7 +499,7 @@ private:
               "        }\n"
               "        return *this\n"
               "    }\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("[test.cpp:6]: (warning) Member variable 'Fred::i' is not assigned a value in 'Fred::operator='.\n", errout.str());
 
         check("class Fred\n"
@@ -399,7 +514,7 @@ private:
               "        }\n"
               "        return *this\n"
               "    }\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("[test.cpp:6]: (warning) Member variable 'Fred::i' is not assigned a value in 'Fred::operator='.\n", errout.str());
 
         check("class Fred\n"
@@ -414,7 +529,7 @@ private:
               "        }\n"
               "        return *this\n"
               "    }\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("[test.cpp:6]: (warning) Member variable 'Fred::i' is not assigned a value in 'Fred::operator='.\n", errout.str());
 
         check("class Fred\n"
@@ -429,7 +544,7 @@ private:
               "        }\n"
               "        return *this\n"
               "    }\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -444,7 +559,7 @@ private:
               "        Fred(rhs).swap(*this);\n"
               "        return *this;\n"
               "    }\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -467,7 +582,7 @@ private:
               "        int b;\n"
               "        Fred() { b = 0; }\n"
               "    };\n"
-              "}\n");
+              "}");
 
         ASSERT_EQUALS("", errout.str());
 
@@ -487,7 +602,7 @@ private:
               "        int b;\n"
               "        Fred() { b = 0; }\n"
               "    };\n"
-              "}\n");
+              "}");
 
         ASSERT_EQUALS("", errout.str());
 
@@ -507,7 +622,7 @@ private:
               "        int b;\n"
               "        Fred() { b = 0; }\n"
               "    };\n"
-              "}\n");
+              "}");
 
         ASSERT_EQUALS("", errout.str());
 
@@ -529,7 +644,7 @@ private:
               "            Fred() { b = 0; }\n"
               "        };\n"
               "    }\n"
-              "};\n");
+              "};");
 
         ASSERT_EQUALS("", errout.str());
 
@@ -551,7 +666,7 @@ private:
               "            Fred() { }\n"
               "        };\n"
               "    }\n"
-              "};\n");
+              "};");
 
         ASSERT_EQUALS("[test.cpp:7]: (warning) Member variable 'Fred::a' is not initialized in the constructor.\n"
                       "[test.cpp:16]: (warning) Member variable 'Fred::b' is not initialized in the constructor.\n", errout.str());
@@ -571,7 +686,7 @@ private:
               "c::c()\n"
               "{\n"
               "    m_iMyInt1 = m_iMyInt2 = 0;\n"
-              "}\n");
+              "}");
 
         ASSERT_EQUALS("", errout.str());
     }
@@ -604,7 +719,7 @@ private:
               "void c::InitInt()\n"
               "{\n"
               "    m_iMyInt = 0;\n"
-              "}\n");
+              "}");
 
         ASSERT_EQUALS("", errout.str());
     }
@@ -646,7 +761,7 @@ private:
               "public:\n"
               "    Fred() { }\n"
               "    static void *p;\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -664,7 +779,7 @@ private:
               "    {\n"
               "        U.a = 0;\n"
               "    }\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("", errout.str());
 
         check("class Fred\n"
@@ -678,11 +793,48 @@ private:
               "    Fred()\n"
               "    {\n"
               "    }\n"
-              "};\n");
+              "};");
         TODO_ASSERT_EQUALS("[test.cpp:9]: (warning) Member variable 'Fred::U' is not initialized in the constructor.\n", "", errout.str());
     }
 
 
+    void initvar_delegate() {
+        check("class A {\n"
+              "    int number;\n"
+              "public:\n"
+              "    A(int n) { }\n"
+              "    A() : A(42) {}\n"
+              "};");
+        ASSERT_EQUALS("[test.cpp:4]: (warning) Member variable 'A::number' is not initialized in the constructor.\n"
+                      "[test.cpp:5]: (warning) Member variable 'A::number' is not initialized in the constructor.\n", errout.str());
+
+        check("class A {\n"
+              "    int number;\n"
+              "public:\n"
+              "    A(int n) { number = n; }\n"
+              "    A() : A(42) {}\n"
+              "};");
+        ASSERT_EQUALS("", errout.str());
+
+        check("class A {\n"
+              "    int number;\n"
+              "public:\n"
+              "    A(int n) : A() { }\n"
+              "    A() {}\n"
+              "};", true);
+        ASSERT_EQUALS("[test.cpp:4]: (warning) Member variable 'A::number' is not initialized in the constructor.\n"
+                      "[test.cpp:5]: (warning, inconclusive) Member variable 'A::number' is not initialized in the constructor.\n", errout.str());
+
+        check("class A {\n"
+              "    int number;\n"
+              "public:\n"
+              "    A(int n) : A() { }\n"
+              "    A() { number = 42; }\n"
+              "};");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+
     void initvar_private_constructor() {
         check("class Fred\n"
               "{\n"
@@ -706,18 +858,6 @@ private:
               "};");
         ASSERT_EQUALS("", errout.str());
 
-        check("class Fred\n"
-              "{\n"
-              "private:\n"
-              "    std::string var;\n"
-              "public:\n"
-              "    Fred();\n"
-              "    Fred(const Fred &);\n"
-              "};\n"
-              "Fred::Fred() { };\n"
-              "Fred::Fred(const Fred &) { };\n");
-        ASSERT_EQUALS("", errout.str());
-
         check("class Fred\n"
               "{\n"
               "private:\n"
@@ -764,7 +904,7 @@ private:
               "A::A(){}\n"
               "A::B::B(int x){}\n"
               "A::B::C::C(int y){}\n"
-              "A::B::C::D::D(int z){}\n");
+              "A::B::C::D::D(int z){}");
         // Note that the example code is not compilable. The A constructor must
         // explicitly initialize A::b. A warning for A::b is not necessary.
         ASSERT_EQUALS("[test.cpp:20]: (warning) Member variable 'A::a' is not initialized in the constructor.\n"
@@ -794,7 +934,7 @@ private:
               "A::A(){}\n"
               "A::B::B(int x){}\n"
               "A::B::C::C(int y){}\n"
-              "A::B::C::D::D(const A::B::C::D & d){}\n");
+              "A::B::C::D::D(const A::B::C::D & d){}");
         // Note that the example code is not compilable. The A constructor must
         // explicitly initialize A::b. A warning for A::b is not necessary.
         ASSERT_EQUALS("[test.cpp:20]: (warning) Member variable 'A::a' is not initialized in the constructor.\n"
@@ -825,7 +965,7 @@ private:
               "A::A(){}\n"
               "A::B::B(int x){}\n"
               "A::B::C::C(int y){}\n"
-              "A::B::C::D::D(const A::B::C::D::E & e){}\n");
+              "A::B::C::D::D(const A::B::C::D::E & e){}");
         // Note that the example code is not compilable. The A constructor must
         // explicitly initialize A::b. A warning for A::b is not necessary.
         ASSERT_EQUALS("[test.cpp:21]: (warning) Member variable 'A::a' is not initialized in the constructor.\n"
@@ -838,7 +978,6 @@ private:
         check("class B\n"
               "{\n"
               "    B (const B & Var);\n"
-              "    B & operator= (const B & Var);\n"
               "};\n"
               "class A\n"
               "{\n"
@@ -846,14 +985,81 @@ private:
               "public:\n"
               "    A(){}\n"
               "    A(const A&){}\n"
+              "    A(A &&){}\n"
               "    const A& operator=(const A&){return *this;}\n"
-              "};\n");
+              "};");
+        ASSERT_EQUALS("", errout.str());
+
+        check("class B\n"
+              "{\n"
+              "    B (B && Var);\n"
+              "};\n"
+              "class A\n"
+              "{\n"
+              "    B m_SemVar;\n"
+              "public:\n"
+              "    A(){}\n"
+              "    A(const A&){}\n"
+              "    A(A &&){}\n"
+              "    const A& operator=(const A&){return *this;}\n"
+              "};");
+        ASSERT_EQUALS("", errout.str());
+
+        check("class B\n"
+              "{\n"
+              "    B & operator= (const B & Var);\n"
+              "public:\n"
+              "    B ();\n"
+              "};\n"
+              "class A\n"
+              "{\n"
+              "    B m_SemVar;\n"
+              "public:\n"
+              "    A(){}\n"
+              "    A(const A&){}\n"
+              "    A(A &&){}\n"
+              "    const A& operator=(const A&){return *this;}\n"
+              "};");
         ASSERT_EQUALS("", errout.str());
 
         check("class B\n"
               "{\n"
               "public:\n"
               "    B (const B & Var);\n"
+              "};\n"
+              "class A\n"
+              "{\n"
+              "    B m_SemVar;\n"
+              "public:\n"
+              "    A(){}\n"
+              "    A(const A&){}\n"
+              "    A(A &&){}\n"
+              "    const A& operator=(const A&){return *this;}\n"
+              "};");
+        ASSERT_EQUALS("[test.cpp:11]: (warning) Member variable 'A::m_SemVar' is not initialized in the constructor.\n"
+                      "[test.cpp:12]: (warning) Member variable 'A::m_SemVar' is not initialized in the constructor.\n"
+                      "[test.cpp:13]: (warning) Member variable 'A::m_SemVar' is not assigned a value in 'A::operator='.\n", errout.str());
+
+        check("class B\n"
+              "{\n"
+              "public:\n"
+              "    B (B && Var);\n"
+              "};\n"
+              "class A\n"
+              "{\n"
+              "    B m_SemVar;\n"
+              "public:\n"
+              "    A(){}\n"
+              "    A(const A&){}\n"
+              "    A(A &&){}\n"
+              "    const A& operator=(const A&){return *this;}\n"
+              "};");
+        ASSERT_EQUALS("[test.cpp:12]: (warning) Member variable 'A::m_SemVar' is not initialized in the constructor.\n", errout.str());
+
+        check("class B\n"
+              "{\n"
+              "public:\n"
+              "    B ();\n"
               "    B & operator= (const B & Var);\n"
               "};\n"
               "class A\n"
@@ -862,10 +1068,12 @@ private:
               "public:\n"
               "    A(){}\n"
               "    A(const A&){}\n"
+              "    A(A &&){}\n"
               "    const A& operator=(const A&){return *this;}\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("[test.cpp:12]: (warning) Member variable 'A::m_SemVar' is not initialized in the constructor.\n"
-                      "[test.cpp:13]: (warning) Member variable 'A::m_SemVar' is not assigned a value in 'A::operator='.\n", errout.str());
+                      "[test.cpp:13]: (warning) Member variable 'A::m_SemVar' is not initialized in the constructor.\n"
+                      "[test.cpp:14]: (warning) Member variable 'A::m_SemVar' is not assigned a value in 'A::operator='.\n", errout.str());
 
         check("class A\n"
               "{\n"
@@ -874,7 +1082,7 @@ private:
               "    A(){}\n"
               "    A(const A&){}\n"
               "    const A& operator=(const A&){return *this;}\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -891,7 +1099,7 @@ private:
               "    A(){}\n"
               "    A(const A&){}\n"
               "    const A& operator=(const A&){return *this;}\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("", errout.str());
 
         check("class B\n"
@@ -907,7 +1115,7 @@ private:
               "    A(){}\n"
               "    A(const A&){}\n"
               "    const A& operator=(const A&){return *this;}\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("[test.cpp:12]: (warning) Member variable 'A::m_SemVar' is not initialized in the constructor.\n"
                       "[test.cpp:13]: (warning) Member variable 'A::m_SemVar' is not assigned a value in 'A::operator='.\n", errout.str());
     }
@@ -928,6 +1136,15 @@ private:
         ASSERT_EQUALS("[test.cpp:4]: (warning, inconclusive) Member variable 'A::b' is not initialized in the constructor.\n", errout.str());
     }
 
+    void initvar_with_member_function_this() {
+        check("struct Foo {\n"
+              "  Foo(int m) { this->setMember(m); }\n"
+              "  void setMember(int m) { member = m; }\n"
+              "  int member;\n"
+              "};");
+        ASSERT_EQUALS("", errout.str());
+    }
+
     void initvar_destructor() {
         check("class Fred\n"
               "{\n"
@@ -936,7 +1153,15 @@ private:
               "public:\n"
               "    Fred() : var(0) {}\n"
               "    ~Fred() {}\n"
-              "};\n");
+              "};");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void initvar_func_ret_func_ptr() { // ticket #4449 (segmentation fault)
+        check("class something {\n"
+              "    int * ( something :: * process()) () { return 0; }\n"
+              "    something() { process(); }\n"
+              "};");
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -1522,7 +1747,7 @@ private:
               "private:\n"
               "    int a[23];\n"
               "public:\n"
-              "    Fred(); \n"
+              "    Fred();\n"
               "};\n"
               "Fred::Fred() {\n"
               "    a[x::y] = 0;\n"
@@ -1561,6 +1786,153 @@ private:
         ASSERT_EQUALS("[test.cpp:7]: (warning) Member variable 'Fred::x' is not assigned a value in 'Fred::operator='.\n", errout.str());
     }
 
+    void uninitVar23() { // ticket #3702
+        check("class Fred {\n"
+              "    int x;\n"
+              "public:\n"
+              "    Fred(struct A a, struct B b);\n"
+              "    Fred(C c, struct D d);\n"
+              "    Fred(struct E e, F f);\n"
+              "    Fred(struct G, struct H);\n"
+              "    Fred(I, J);\n"
+              "};\n"
+              "Fred::Fred(A a, B b) { }\n"
+              "Fred::Fred(struct C c, D d) { }\n"
+              "Fred::Fred(E e, struct F f) { }\n"
+              "Fred::Fred(G g, H h) { }\n"
+              "Fred::Fred(struct I i, struct J j) { }\n"
+             );
+        ASSERT_EQUALS("[test.cpp:10]: (warning) Member variable 'Fred::x' is not initialized in the constructor.\n"
+                      "[test.cpp:11]: (warning) Member variable 'Fred::x' is not initialized in the constructor.\n"
+                      "[test.cpp:12]: (warning) Member variable 'Fred::x' is not initialized in the constructor.\n"
+                      "[test.cpp:13]: (warning) Member variable 'Fred::x' is not initialized in the constructor.\n"
+                      "[test.cpp:14]: (warning) Member variable 'Fred::x' is not initialized in the constructor.\n", errout.str());
+    }
+
+    void uninitVar24() { // ticket #3190
+        check("class Foo;\n"
+              "class Bar;\n"
+              "class Sub;\n"
+              "class Foo { class Sub; };\n"
+              "class Bar { class Sub; };\n"
+              "class Bar::Sub {\n"
+              "    int b;\n"
+              "public:\n"
+              "    Sub() { }\n"
+              "    Sub(int);\n"
+              "};\n"
+              "Bar::Sub::Sub(int) { };\n"
+              "class ::Foo::Sub {\n"
+              "    int f;\n"
+              "public:\n"
+              "    ~Sub();\n"
+              "    Sub();\n"
+              "};\n"
+              "::Foo::Sub::~Sub() { }\n"
+              "::Foo::Sub::Sub() { }\n"
+              "class Foo;\n"
+              "class Bar;\n"
+              "class Sub;\n", true);
+
+        ASSERT_EQUALS("[test.cpp:9]: (warning, inconclusive) Member variable 'Sub::b' is not initialized in the constructor.\n"
+                      "[test.cpp:12]: (warning) Member variable 'Sub::b' is not initialized in the constructor.\n"
+                      "[test.cpp:20]: (warning) Member variable 'Sub::f' is not initialized in the constructor.\n", errout.str());
+    }
+
+    void uninitVar25() { // ticket #4789
+        check("struct A {\n"
+              "    int a;\n"
+              "    int b;\n"
+              "    int c;\n"
+              "    A(int x = 0, int y = 0, int z = 0);\n"
+              "};\n"
+              "A::A(int x = 0, int y = 0, int z = 0) { } \n"
+              "struct B {\n"
+              "    int a;\n"
+              "    int b;\n"
+              "    int c;\n"
+              "    B(int x = 0, int y = 0, int z = 0);\n"
+              "};\n"
+              "B::B(int x, int y, int z) { } \n"
+              "struct C {\n"
+              "    int a;\n"
+              "    int b;\n"
+              "    int c;\n"
+              "    C(int, int, int);\n"
+              "};\n"
+              "C::C(int x = 0, int y = 0, int z = 0) { } \n"
+              "struct D {\n"
+              "    int a;\n"
+              "    int b;\n"
+              "    int c;\n"
+              "    D(int, int, int);\n"
+              "};\n"
+              "D::D(int x, int y, int z) { } \n"
+              "struct E {\n"
+              "    int a;\n"
+              "    int b;\n"
+              "    int c;\n"
+              "    E(int x, int y, int z);\n"
+              "};\n"
+              "E::E(int, int, int) { } \n"
+              "struct F {\n"
+              "    int a;\n"
+              "    int b;\n"
+              "    int c;\n"
+              "    F(int x = 0, int y = 0, int z = 0);\n"
+              "};\n"
+              "F::F(int, int, int) { }\n", true);
+        ASSERT_EQUALS("[test.cpp:7]: (warning) Member variable 'A::a' is not initialized in the constructor.\n"
+                      "[test.cpp:7]: (warning) Member variable 'A::b' is not initialized in the constructor.\n"
+                      "[test.cpp:7]: (warning) Member variable 'A::c' is not initialized in the constructor.\n"
+                      "[test.cpp:14]: (warning) Member variable 'B::a' is not initialized in the constructor.\n"
+                      "[test.cpp:14]: (warning) Member variable 'B::b' is not initialized in the constructor.\n"
+                      "[test.cpp:14]: (warning) Member variable 'B::c' is not initialized in the constructor.\n"
+                      "[test.cpp:21]: (warning) Member variable 'C::a' is not initialized in the constructor.\n"
+                      "[test.cpp:21]: (warning) Member variable 'C::b' is not initialized in the constructor.\n"
+                      "[test.cpp:21]: (warning) Member variable 'C::c' is not initialized in the constructor.\n"
+                      "[test.cpp:28]: (warning) Member variable 'D::a' is not initialized in the constructor.\n"
+                      "[test.cpp:28]: (warning) Member variable 'D::b' is not initialized in the constructor.\n"
+                      "[test.cpp:28]: (warning) Member variable 'D::c' is not initialized in the constructor.\n"
+                      "[test.cpp:35]: (warning) Member variable 'E::a' is not initialized in the constructor.\n"
+                      "[test.cpp:35]: (warning) Member variable 'E::b' is not initialized in the constructor.\n"
+                      "[test.cpp:35]: (warning) Member variable 'E::c' is not initialized in the constructor.\n"
+                      "[test.cpp:42]: (warning) Member variable 'F::a' is not initialized in the constructor.\n"
+                      "[test.cpp:42]: (warning) Member variable 'F::b' is not initialized in the constructor.\n"
+                      "[test.cpp:42]: (warning) Member variable 'F::c' is not initialized in the constructor.\n", errout.str());
+    }
+
+    void uninitVar26() {
+        check("class A {\n"
+              "    int * v;\n"
+              "    int sz;\n"
+              "public:\n"
+              "    A(int s) {\n"
+              "        v = new int [sz = s];\n"
+              "    }\n"
+              "};");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void uninitVar27() {
+        check("class A {\n"
+              "    double d;\n"
+              "public:\n"
+              "    A() {\n"
+              "        rtl::math::setNan(&d);\n"
+              "    }\n"
+              "};");
+        ASSERT_EQUALS("", errout.str());
+        check("class A {\n"
+              "    double d;\n"
+              "public:\n"
+              "    A() {\n"
+              "        ::rtl::math::setNan(&d);\n"
+              "    }\n"
+              "};");
+        ASSERT_EQUALS("", errout.str());
+    }
+
     void uninitVarArray1() {
         check("class John\n"
               "{\n"
@@ -1585,7 +1957,7 @@ private:
         check("class John\n"
               "{\n"
               "public:\n"
-              "    John() { strcpy(name, ""); }\n"
+              "    John() { strcpy(name, \"\"); }\n"
               "\n"
               "private:\n"
               "    char name[255];\n"
@@ -1748,7 +2120,7 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
-    void uninitVarCpp11Init() {
+    void uninitVarCpp11Init1() {
         check("class Foo {\n"
               "    std::vector<std::string> bar;\n"
               "public:\n"
@@ -1759,6 +2131,22 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void uninitVarCpp11Init2() {
+        check("class Fred {\n"
+              "    struct Foo {\n"
+              "        int a;\n"
+              "        bool b;\n"
+              "    };\n"
+              "    Foo f;\n"
+              "    float g;\n"
+              "public:\n"
+              "    Fred() : f{0, true} { }\n"
+              "    float get() const\n"
+              "};\n"
+              "float Fred::get() const { return g; }");
+        ASSERT_EQUALS("[test.cpp:9]: (warning) Member variable 'Fred::g' is not initialized in the constructor.\n", errout.str());
+    }
+
     void uninitVarStruct1() { // ticket #2172
         check("class A\n"
               "{\n"
@@ -2373,7 +2761,7 @@ private:
               "    void init(int value)\n"
               "    { }\n"
               "};");
-        TODO_ASSERT_EQUALS("[test.cpp:7]: (warning) Member variable 'A::i' is not initialized in the constructor.\n", "", errout.str());
+        ASSERT_EQUALS("[test.cpp:7]: (warning) Member variable 'A::i' is not initialized in the constructor.\n", errout.str());
     }
 
     void uninitVarOperatorEqual() { // ticket #2415
