@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,10 +23,12 @@
 
 class TestErrorLogger : public TestFixture {
 public:
-    TestErrorLogger() : TestFixture("TestErrorLogger")
-    { }
+    TestErrorLogger() : TestFixture("TestErrorLogger"), fooCpp5("foo.cpp", 5), barCpp8("bar.cpp", 8) {
+    }
 
 private:
+    const ErrorLogger::ErrorMessage::FileLocation fooCpp5;
+    const ErrorLogger::ErrorMessage::FileLocation barCpp8;
 
     void run() {
         TEST_CASE(FileLocationDefaults);
@@ -54,25 +56,21 @@ private:
         TEST_CASE(suppressUnmatchedSuppressions);
     }
 
-    void FileLocationDefaults() {
+    void FileLocationDefaults() const {
         ErrorLogger::ErrorMessage::FileLocation loc;
         ASSERT_EQUALS("", loc.getfile());
         ASSERT_EQUALS(0, loc.line);
     }
 
-    void FileLocationSetFile() {
+    void FileLocationSetFile() const {
         ErrorLogger::ErrorMessage::FileLocation loc;
         loc.setfile("foo.cpp");
         ASSERT_EQUALS("foo.cpp", loc.getfile());
         ASSERT_EQUALS(0, loc.line);
     }
 
-    void ErrorMessageConstruct() {
-        ErrorLogger::ErrorMessage::FileLocation loc;
-        loc.setfile("foo.cpp");
-        loc.line = 5;
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-        locs.push_back(loc);
+    void ErrorMessageConstruct() const {
+        std::list<ErrorLogger::ErrorMessage::FileLocation> locs(1, fooCpp5);
         ErrorMessage msg(locs, Severity::error, "Programming error.", "errorId", false);
         ASSERT_EQUALS(1, (int)msg._callStack.size());
         ASSERT_EQUALS("Programming error.", msg.shortMessage());
@@ -81,16 +79,10 @@ private:
         ASSERT_EQUALS("[foo.cpp:5]: (error) Programming error.", msg.toString(true));
     }
 
-    void ErrorMessageConstructLocations() {
-        ErrorLogger::ErrorMessage::FileLocation loc;
-        loc.setfile("foo.cpp");
-        loc.line = 5;
-        ErrorLogger::ErrorMessage::FileLocation loc2;
-        loc2.setfile("bar.cpp");
-        loc2.line = 8;
+    void ErrorMessageConstructLocations() const {
         std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-        locs.push_back(loc);
-        locs.push_back(loc2);
+        locs.push_back(fooCpp5);
+        locs.push_back(barCpp8);
         ErrorMessage msg(locs, Severity::error, "Programming error.", "errorId", false);
         ASSERT_EQUALS(2, (int)msg._callStack.size());
         ASSERT_EQUALS("Programming error.", msg.shortMessage());
@@ -99,12 +91,8 @@ private:
         ASSERT_EQUALS("[foo.cpp:5] -> [bar.cpp:8]: (error) Programming error.", msg.toString(true));
     }
 
-    void ErrorMessageVerbose() {
-        ErrorLogger::ErrorMessage::FileLocation loc;
-        loc.setfile("foo.cpp");
-        loc.line = 5;
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-        locs.push_back(loc);
+    void ErrorMessageVerbose() const {
+        std::list<ErrorLogger::ErrorMessage::FileLocation> locs(1, fooCpp5);
         ErrorMessage msg(locs, Severity::error, "Programming error.\nVerbose error", "errorId", false);
         ASSERT_EQUALS(1, (int)msg._callStack.size());
         ASSERT_EQUALS("Programming error.", msg.shortMessage());
@@ -113,16 +101,10 @@ private:
         ASSERT_EQUALS("[foo.cpp:5]: (error) Verbose error", msg.toString(true));
     }
 
-    void ErrorMessageVerboseLocations() {
-        ErrorLogger::ErrorMessage::FileLocation loc;
-        loc.setfile("foo.cpp");
-        loc.line = 5;
+    void ErrorMessageVerboseLocations() const {
         std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-        ErrorLogger::ErrorMessage::FileLocation loc2;
-        loc2.setfile("bar.cpp");
-        loc2.line = 8;
-        locs.push_back(loc);
-        locs.push_back(loc2);
+        locs.push_back(fooCpp5);
+        locs.push_back(barCpp8);
         ErrorMessage msg(locs, Severity::error, "Programming error.\nVerbose error", "errorId", false);
         ASSERT_EQUALS(2, (int)msg._callStack.size());
         ASSERT_EQUALS("Programming error.", msg.shortMessage());
@@ -131,12 +113,8 @@ private:
         ASSERT_EQUALS("[foo.cpp:5] -> [bar.cpp:8]: (error) Verbose error", msg.toString(true));
     }
 
-    void CustomFormat() {
-        ErrorLogger::ErrorMessage::FileLocation loc;
-        loc.setfile("foo.cpp");
-        loc.line = 5;
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-        locs.push_back(loc);
+    void CustomFormat() const {
+        std::list<ErrorLogger::ErrorMessage::FileLocation> locs(1, fooCpp5);
         ErrorMessage msg(locs, Severity::error, "Programming error.\nVerbose error", "errorId", false);
         ASSERT_EQUALS(1, (int)msg._callStack.size());
         ASSERT_EQUALS("Programming error.", msg.shortMessage());
@@ -145,12 +123,8 @@ private:
         ASSERT_EQUALS("foo.cpp:5,error,errorId,Verbose error", msg.toString(true, "{file}:{line},{severity},{id},{message}"));
     }
 
-    void CustomFormat2() {
-        ErrorLogger::ErrorMessage::FileLocation loc;
-        loc.setfile("foo.cpp");
-        loc.line = 5;
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-        locs.push_back(loc);
+    void CustomFormat2() const {
+        std::list<ErrorLogger::ErrorMessage::FileLocation> locs(1, fooCpp5);
         ErrorMessage msg(locs, Severity::error, "Programming error.\nVerbose error", "errorId", false);
         ASSERT_EQUALS(1, (int)msg._callStack.size());
         ASSERT_EQUALS("Programming error.", msg.shortMessage());
@@ -159,17 +133,11 @@ private:
         ASSERT_EQUALS("Verbose error - foo.cpp(5):(error,errorId)", msg.toString(true, "{message} - {file}({line}):({severity},{id})"));
     }
 
-    void CustomFormatLocations() {
+    void CustomFormatLocations() const {
         // Check that first location from location stack is used in template
-        ErrorLogger::ErrorMessage::FileLocation loc;
-        loc.setfile("foo.cpp");
-        loc.line = 5;
-        ErrorLogger::ErrorMessage::FileLocation loc2;
-        loc2.setfile("bar.cpp");
-        loc2.line = 8;
         std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-        locs.push_back(loc);
-        locs.push_back(loc2);
+        locs.push_back(fooCpp5);
+        locs.push_back(barCpp8);
         ErrorMessage msg(locs, Severity::error, "Programming error.\nVerbose error", "errorId", false);
         ASSERT_EQUALS(2, (int)msg._callStack.size());
         ASSERT_EQUALS("Programming error.", msg.shortMessage());
@@ -178,112 +146,78 @@ private:
         ASSERT_EQUALS("Verbose error - bar.cpp(8):(error,errorId)", msg.toString(true, "{message} - {file}({line}):({severity},{id})"));
     }
 
-    void ToXml() {
-        ErrorLogger::ErrorMessage::FileLocation loc;
-        loc.setfile("foo.cpp");
-        loc.line = 5;
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-        locs.push_back(loc);
+    void ToXml() const {
+        std::list<ErrorLogger::ErrorMessage::FileLocation> locs(1, fooCpp5);
         ErrorMessage msg(locs, Severity::error, "Programming error.\nVerbose error", "errorId", false);
         ASSERT_EQUALS("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<results>", ErrorLogger::ErrorMessage::getXMLHeader(1));
         ASSERT_EQUALS("</results>", ErrorLogger::ErrorMessage::getXMLFooter(1));
-        ASSERT_EQUALS("<error file=\"foo.cpp\" line=\"5\" id=\"errorId\" severity=\"error\" msg=\"Programming error.\"/>", msg.toXML(false,1));
+        ASSERT_EQUALS("    <error file=\"foo.cpp\" line=\"5\" id=\"errorId\" severity=\"error\" msg=\"Programming error.\"/>", msg.toXML(false,1));
     }
 
-    void ToXmlLocations() {
-        ErrorLogger::ErrorMessage::FileLocation loc;
-        loc.setfile("foo.cpp");
-        loc.line = 5;
-        ErrorLogger::ErrorMessage::FileLocation loc2;
-        loc2.setfile("bar.cpp");
-        loc2.line = 8;
+    void ToXmlLocations() const {
         std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-        locs.push_back(loc);
-        locs.push_back(loc2);
+        locs.push_back(fooCpp5);
+        locs.push_back(barCpp8);
         ErrorMessage msg(locs, Severity::error, "Programming error.\nVerbose error", "errorId", false);
         ASSERT_EQUALS("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<results>", ErrorLogger::ErrorMessage::getXMLHeader(1));
         ASSERT_EQUALS("</results>", ErrorLogger::ErrorMessage::getXMLFooter(1));
-        ASSERT_EQUALS("<error file=\"bar.cpp\" line=\"8\" id=\"errorId\" severity=\"error\" msg=\"Programming error.\"/>", msg.toXML(false,1));
+        ASSERT_EQUALS("    <error file=\"bar.cpp\" line=\"8\" id=\"errorId\" severity=\"error\" msg=\"Programming error.\"/>", msg.toXML(false,1));
     }
 
-    void ToVerboseXml() {
-        ErrorLogger::ErrorMessage::FileLocation loc;
-        loc.setfile("foo.cpp");
-        loc.line = 5;
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-        locs.push_back(loc);
+    void ToVerboseXml() const {
+        std::list<ErrorLogger::ErrorMessage::FileLocation> locs(1, fooCpp5);
         ErrorMessage msg(locs, Severity::error, "Programming error.\nVerbose error", "errorId", false);
         ASSERT_EQUALS("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<results>", ErrorLogger::ErrorMessage::getXMLHeader(1));
         ASSERT_EQUALS("</results>", ErrorLogger::ErrorMessage::getXMLFooter(1));
-        ASSERT_EQUALS("<error file=\"foo.cpp\" line=\"5\" id=\"errorId\" severity=\"error\" msg=\"Verbose error\"/>", msg.toXML(true,1));
+        ASSERT_EQUALS("    <error file=\"foo.cpp\" line=\"5\" id=\"errorId\" severity=\"error\" msg=\"Verbose error\"/>", msg.toXML(true,1));
     }
 
-    void ToVerboseXmlLocations() {
-        ErrorLogger::ErrorMessage::FileLocation loc;
-        loc.setfile("foo.cpp");
-        loc.line = 5;
-        ErrorLogger::ErrorMessage::FileLocation loc2;
-        loc2.setfile("bar.cpp");
-        loc2.line = 8;
+    void ToVerboseXmlLocations() const {
         std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-        locs.push_back(loc);
-        locs.push_back(loc2);
+        locs.push_back(fooCpp5);
+        locs.push_back(barCpp8);
         ErrorMessage msg(locs, Severity::error, "Programming error.\nVerbose error", "errorId", false);
         ASSERT_EQUALS("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<results>", ErrorLogger::ErrorMessage::getXMLHeader(1));
         ASSERT_EQUALS("</results>", ErrorLogger::ErrorMessage::getXMLFooter(1));
-        ASSERT_EQUALS("<error file=\"bar.cpp\" line=\"8\" id=\"errorId\" severity=\"error\" msg=\"Verbose error\"/>", msg.toXML(true,1));
+        ASSERT_EQUALS("    <error file=\"bar.cpp\" line=\"8\" id=\"errorId\" severity=\"error\" msg=\"Verbose error\"/>", msg.toXML(true,1));
     }
 
-    void ToXmlV2() {
-        ErrorLogger::ErrorMessage::FileLocation loc;
-        loc.setfile("foo.cpp");
-        loc.line = 5;
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-        locs.push_back(loc);
+    void ToXmlV2() const {
+        std::list<ErrorLogger::ErrorMessage::FileLocation> locs(1, fooCpp5);
         ErrorMessage msg(locs, Severity::error, "Programming error.\nVerbose error", "errorId", false);
         std::string header("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<results version=\"2\">\n");
-        header += "  <cppcheck version=\"";
+        header += "    <cppcheck version=\"";
         header += CppCheck::version();
-        header += "\"/>\n  <errors>";
+        header += "\"/>\n    <errors>";
         ASSERT_EQUALS(header, ErrorLogger::ErrorMessage::getXMLHeader(2));
-        ASSERT_EQUALS("  </errors>\n</results>", ErrorLogger::ErrorMessage::getXMLFooter(2));
-        std::string message("  <error id=\"errorId\" severity=\"error\"");
+        ASSERT_EQUALS("    </errors>\n</results>", ErrorLogger::ErrorMessage::getXMLFooter(2));
+        std::string message("        <error id=\"errorId\" severity=\"error\"");
         message += " msg=\"Programming error.\" verbose=\"Verbose error\">\n";
-        message += "    <location file=\"foo.cpp\" line=\"5\"/>\n  </error>";
+        message += "            <location file=\"foo.cpp\" line=\"5\"/>\n        </error>";
         ASSERT_EQUALS(message, msg.toXML(false, 2));
     }
 
-    void ToXmlV2Locations() {
-        ErrorLogger::ErrorMessage::FileLocation loc;
-        loc.setfile("foo.cpp");
-        loc.line = 5;
-        ErrorLogger::ErrorMessage::FileLocation loc2;
-        loc2.setfile("bar.cpp");
-        loc2.line = 8;
+    void ToXmlV2Locations() const {
         std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-        locs.push_back(loc);
-        locs.push_back(loc2);
+        locs.push_back(fooCpp5);
+        locs.push_back(barCpp8);
         ErrorMessage msg(locs, Severity::error, "Programming error.\nVerbose error", "errorId", false);
         std::string header("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<results version=\"2\">\n");
-        header += "  <cppcheck version=\"";
+        header += "    <cppcheck version=\"";
         header += CppCheck::version();
-        header += "\"/>\n  <errors>";
+        header += "\"/>\n    <errors>";
         ASSERT_EQUALS(header, ErrorLogger::ErrorMessage::getXMLHeader(2));
-        ASSERT_EQUALS("  </errors>\n</results>", ErrorLogger::ErrorMessage::getXMLFooter(2));
-        std::string message("  <error id=\"errorId\" severity=\"error\"");
+        ASSERT_EQUALS("    </errors>\n</results>", ErrorLogger::ErrorMessage::getXMLFooter(2));
+        std::string message("        <error id=\"errorId\" severity=\"error\"");
         message += " msg=\"Programming error.\" verbose=\"Verbose error\">\n";
-        message += "    <location file=\"bar.cpp\" line=\"8\"/>\n";
-        message += "    <location file=\"foo.cpp\" line=\"5\"/>\n  </error>";
+        message += "            <location file=\"bar.cpp\" line=\"8\"/>\n";
+        message += "            <location file=\"foo.cpp\" line=\"5\"/>\n        </error>";
         ASSERT_EQUALS(message, msg.toXML(false, 2));
     }
 
-    void InconclusiveXml() {
+    void InconclusiveXml() const {
         // Location
-        ErrorLogger::ErrorMessage::FileLocation loc;
-        loc.setfile("foo.cpp");
-        loc.line = 5;
-        std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
-        locs.push_back(loc);
+        std::list<ErrorLogger::ErrorMessage::FileLocation> locs(1, fooCpp5);
 
         // Inconclusive error message
         ErrorMessage msg(locs, Severity::error, "Programming error", "errorId", true);
@@ -292,13 +226,13 @@ private:
         ASSERT_EQUALS("", msg.toXML(false, 1));
 
         // xml version 2 error message
-        ASSERT_EQUALS("  <error id=\"errorId\" severity=\"error\" msg=\"Programming error\" verbose=\"Programming error\" inconclusive=\"true\">\n"
-                      "    <location file=\"foo.cpp\" line=\"5\"/>\n"
-                      "  </error>",
+        ASSERT_EQUALS("        <error id=\"errorId\" severity=\"error\" msg=\"Programming error\" verbose=\"Programming error\" inconclusive=\"true\">\n"
+                      "            <location file=\"foo.cpp\" line=\"5\"/>\n"
+                      "        </error>",
                       msg.toXML(false, 2));
     }
 
-    void SerializeInconclusiveMessage() {
+    void SerializeInconclusiveMessage() const {
         // Inconclusive error message
         std::list<ErrorLogger::ErrorMessage::FileLocation> locs;
         ErrorMessage msg(locs, Severity::error, "Programming error", "errorId", true);
