@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 
 #include "testsuite.h"
 #include "filelister.h"
+#include "settings.h"
 #include <fstream>
 
 #ifndef _WIN32
@@ -31,8 +32,8 @@
 class TestFileLister: public TestFixture {
 public:
     TestFileLister()
-        :TestFixture("TestFileLister")
-    {}
+        :TestFixture("TestFileLister") {
+    }
 
 private:
     void run() {
@@ -75,7 +76,8 @@ private:
     void recursiveAddFiles() const {
         // Recursively add add files..
         std::map<std::string, std::size_t> files;
-        FileLister::recursiveAddFiles(files, ".");
+        std::set<std::string> extra;
+        FileLister::recursiveAddFiles(files, ".", extra);
 
         // In case there are leading "./"..
         for (std::map<std::string, std::size_t>::iterator i = files.begin(); i != files.end();) {
