@@ -23,8 +23,8 @@
 
 class TestMathLib : public TestFixture {
 public:
-    TestMathLib() : TestFixture("TestMathLib")
-    { }
+    TestMathLib() : TestFixture("TestMathLib") {
+    }
 
 private:
 
@@ -34,6 +34,7 @@ private:
         TEST_CASE(convert);
         TEST_CASE(isint);
         TEST_CASE(isnegative);
+        TEST_CASE(ispositive);
         TEST_CASE(isfloat);
         TEST_CASE(isGreater)
         TEST_CASE(isGreaterEqual)
@@ -41,6 +42,7 @@ private:
         TEST_CASE(isNotEqual)
         TEST_CASE(isLess)
         TEST_CASE(isLessEqual)
+        TEST_CASE(naninf)
     }
 
     void isGreater() const {
@@ -124,6 +126,7 @@ private:
         ASSERT_EQUALS("7.0" , MathLib::divide("21.", "3"));
         ASSERT_EQUALS("1"   , MathLib::divide("3", "2"));
         ASSERT_THROW(MathLib::divide("123", "0"), InternalError); // throw
+        ASSERT_THROW(MathLib::divide("-9223372036854775808", "-1"), InternalError); // #4520 - out of range => throw
         MathLib::divide("123", "0.0"); // don't throw
 
         // Unknown action should throw exception
@@ -217,6 +220,19 @@ private:
         ASSERT_EQUALS_DOUBLE(0.0   , MathLib::toDoubleNumber("-0.0"));
         ASSERT_EQUALS_DOUBLE(0.0   , MathLib::toDoubleNumber("+0.0"));
 
+        // verify: string --> double --> string conversion
+        ASSERT_EQUALS("1.0" , MathLib::toString(MathLib::toDoubleNumber("1.0f")));
+        ASSERT_EQUALS("1.0" , MathLib::toString(MathLib::toDoubleNumber("1.0")));
+        ASSERT_EQUALS("0.0" , MathLib::toString(MathLib::toDoubleNumber("0.0f")));
+        ASSERT_EQUALS("0.0" , MathLib::toString(MathLib::toDoubleNumber("0.0")));
+        ASSERT_EQUALS("-1.0" , MathLib::toString(MathLib::toDoubleNumber("-1.0f")));
+        ASSERT_EQUALS("-1.0" , MathLib::toString(MathLib::toDoubleNumber("-1.0")));
+        ASSERT_EQUALS("0.0" , MathLib::toString(MathLib::toDoubleNumber("-0.0f")));
+        ASSERT_EQUALS("0.0" , MathLib::toString(MathLib::toDoubleNumber("-0.0")));
+        ASSERT_EQUALS("1.0" , MathLib::toString(MathLib::toDoubleNumber("+1.0f")));
+        ASSERT_EQUALS("1.0" , MathLib::toString(MathLib::toDoubleNumber("+1.0")));
+        ASSERT_EQUALS("0.0" , MathLib::toString(MathLib::toDoubleNumber("+0.0f")));
+        ASSERT_EQUALS("0.0" , MathLib::toString(MathLib::toDoubleNumber("+0.0")));
     }
 
     void isint() const {
@@ -310,6 +326,20 @@ private:
         ASSERT_EQUALS(false, MathLib::isNegative("+1.0E-2"));
     }
 
+    void ispositive() const {
+        ASSERT_EQUALS(false, MathLib::isPositive("-1"));
+        ASSERT_EQUALS(false, MathLib::isPositive("-1."));
+        ASSERT_EQUALS(false, MathLib::isPositive("-1.0"));
+        ASSERT_EQUALS(false, MathLib::isPositive("-1.0E+2"));
+        ASSERT_EQUALS(false, MathLib::isPositive("-1.0E-2"));
+
+        ASSERT_EQUALS(true , MathLib::isPositive("+1"));
+        ASSERT_EQUALS(true , MathLib::isPositive("+1."));
+        ASSERT_EQUALS(true , MathLib::isPositive("+1.0"));
+        ASSERT_EQUALS(true , MathLib::isPositive("+1.0E+2"));
+        ASSERT_EQUALS(true , MathLib::isPositive("+1.0E-2"));
+    }
+
     void isfloat() const {
         ASSERT_EQUALS(false, MathLib::isFloat("0"));
         ASSERT_EQUALS(true , MathLib::isFloat("0."));
@@ -343,6 +373,12 @@ private:
         ASSERT_EQUALS(true , MathLib::isFloat("1.0E-1"));
         ASSERT_EQUALS(true , MathLib::isFloat("-1.0E+1"));
     }
+
+    void naninf() {
+        ASSERT_EQUALS("nan.0", MathLib::divide("0.0", "0.0")); // nan
+        ASSERT_EQUALS("inf.0", MathLib::divide("3.0", "0.0")); // inf
+        ASSERT_EQUALS("-inf.0", MathLib::divide("-3.0", "0.0")); // -inf (#5142)
+    }
 };
 
 REGISTER_TEST(TestMathLib)
